@@ -1,6 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import division, absolute_import
 
+import copy
 import numpy as np
 import astropy.constants as cs
 import scipy.integrate as integ
@@ -197,9 +198,9 @@ class AstroSpectrum(object):
         self.wave = self.wave.to_value(PANDEIA_WAVEUNITS)
         self.flux = self.flux.to_value(PANDEIA_FLUXUNITS)
 
-    def export_to_fits_table(self, fitsfile='source_spectrum.fits'):
-        c1 = fits.Column(array=self.wave, format='D', name='wavelength')
-        c2 = fits.Column(array=self.flux, format='D', name='flux')
+    def export_to_fits_table(self, fitsfile="source_spectrum.fits"):
+        c1 = fits.Column(array=self.wave, format="D", name="wavelength", unit="micron")
+        c2 = fits.Column(array=self.flux, format="D", name="flux", unit="mJy")
         tbhdu = fits.BinTableHDU.from_columns([c1, c2])
         tbhdu.writeto(fitsfile, overwrite=True)
 
@@ -283,8 +284,10 @@ class AstroSpectrum(object):
 
         trim_wave = np.array(trim_wave)
 
-        # this will use synphot to properly sample the endpoints
-        trim_flux = spectrum_resample(self.flux, self.wave, trim_wave)
+        # our spectra are in mJy, which is a flux DENSITY, so interpolation is appropriate for 
+        # trimming (JETC-2812)
+        int_spec = interp.interp1d(self.wave, self.flux, bounds_error=False, fill_value=np.nan)
+        trim_flux = int_spec(trim_wave)
 
         self.wave = trim_wave
         self.flux = trim_flux
@@ -488,6 +491,7 @@ class ConvolvedSceneCube(object):
         mins = []
         maxes = []
         key = None
+        self.filter_leak = []
         for i, src in enumerate(scene.sources):
             spectrum = AstroSpectrum(src, webapp=webapp)
             self.warnings.update(spectrum.warnings)
@@ -511,6 +515,7 @@ class ConvolvedSceneCube(object):
                     key = "wavelength_truncated_red"
                     msg = warning_messages[key] % (smax, wrange['wmax'])
                     self.warnings["%s_%s" % (key, i)] = msg
+
             spectra.append(spectrum)
 
             mins.append(smin)
@@ -538,10 +543,30 @@ class ConvolvedSceneCube(object):
         Trim spectrum and do the spectral convolution here on a per-spectrum basis.  Most efficient to do it here
         before the wavelength sets are merged.  Also easier and much more efficient than convolving
         an axis of a 3D cube.
+        Also compute the filter leak (only valid for image projections), which needs pre- and 
+        post-trimming products.
         """
-        for spectrum in spectra:
+        for i, spectrum in enumerate(spectra):
+            if self.instrument.projection_type in ["image", "image_scan"]:
+                # get the pre-trimming filter leak components
+                full_filtered = self.focal_plane_rate(spectrum.flux, spectrum.wave)
+                full = integ.simps(full_filtered, spectrum.wave)
+
             # we trim here as an optimization so that we only convolve the section we need of a possibly very large spectrum
             spectrum.trim(wrange['wmin'], wrange['wmax'])
+
+            if self.instrument.projection_type in ["image", "image_scan"]:
+                # get the post-trimming filter leak components
+                trim_filtered = self.focal_plane_rate(spectrum.flux, spectrum.wave)
+                trim = integ.simps(trim_filtered, spectrum.wave)
+
+                # compute filter leak and set warning if applicable
+                filter_leak = (full-trim)/full
+                if filter_leak > self.instrument.max_filter_leak:
+                    key = "filter_leak"
+                    msg = warning_messages[key].format(filter_leak*100)
+                    self.warnings["%s_%s" % (key, i)] = msg
+
             spectrum = instrument.spectrometer_convolve(spectrum)
             self.source_spectra.append(spectrum)
         # include the background in the wavelength array calculation, so it's eventually properly sampled
@@ -569,7 +594,7 @@ class ConvolvedSceneCube(object):
         whether the pixel sampling will give us a speed advantage. Otherwise, do not resample to
         an unnecessarily fine wavelength grid.
         """
-        if projection_type in ('spec', 'slitless', 'multiorder'):
+        if projection_type in ('spec', 'slitless', 'slitless_scan', 'multiorder'):
             wave_pix = instrument.get_wave_pix()
             wave_pix_trim = wave_pix[np.where(np.logical_and(wave_pix >= wrange['wmin'],
                                                              wave_pix <= wrange['wmax']))]
@@ -581,7 +606,7 @@ class ConvolvedSceneCube(object):
         a reasonable number of wavelength bins if necessary. This helps keep the cube rendering reasonable
         for large input spectra. Note that the spectrum resampling now uses synphot's flux conserving method
         """
-        if projection_type == 'image':
+        if projection_type in ('image', 'image_scan'):
             if self.wave.size > SPECTRAL_MAX_SAMPLES:
                 self.wave = np.linspace(wrange['wmin'], wrange['wmax'], SPECTRAL_MAX_SAMPLES)
 
@@ -666,6 +691,20 @@ class ConvolvedSceneCube(object):
 
         return fov_size
 
+    def focal_plane_rate(self, rate, wave):
+        """
+        Takes the output from self.ote_rate() and multiplies it by the components of efficiency within the
+        system and returns the source rate at the focal plane in e-/s/pixel/micron.
+        """
+        filter_eff = self.instrument.get_filter_eff(wave)
+        disperser_eff = self.instrument.get_disperser_eff(wave)
+        internal_eff = self.instrument.get_internal_eff(wave)
+        qe = self.instrument.get_detector_qe(wave)
+
+        fp_rate = rate * filter_eff * disperser_eff * internal_eff * qe
+
+        return fp_rate
+
     def create_flux_cube(self, background=None):
         """
         Generate the list of convolved flux cubes that will go into the ETC calculation.
@@ -691,10 +730,12 @@ class ConvolvedSceneCube(object):
         if self.psf_library is None:
             raise NotImplementedError('The use of a simple PSF in isolation is deprecated. Please provide PSFLibrary.')
 
+        ### 1. Load information about PSFs and scene
         instrument_name = self.instrument.get_name()
         aperture_name = self.instrument.get_aperture()
         psf_pixsize_x, psf_pixsize_y = self.psf_library.get_pix_scale(instrument_name, aperture_name)
         psf_upsamp = self.psf_library.get_upsamp(instrument_name, aperture_name)
+        kernel_npix_x, kernel_npix_y = self.psf_library.get_shape(instrument_name, aperture_name)
 
         detector_npix_x = int(np.round(self.fov_size / psf_pixsize_x / psf_upsamp))
         detector_npix_y = int(np.round(self.fov_size / psf_pixsize_y / psf_upsamp))
@@ -703,35 +744,40 @@ class ConvolvedSceneCube(object):
             detector_npix_x += 1
         if detector_npix_y % 2 == 0:
             detector_npix_y += 1
-        detector_shape = (detector_npix_x, detector_npix_y)
+        detector_shape = (detector_npix_y, detector_npix_x)
+
+        ### 2. Create the cubes to fill
         # The cube has to be in a y,x orientation
         # see coord.py Grid definition for more explanation
         flux_cube_list = [
             np.zeros(
-                (int(detector_shape[1]),
-                 int(detector_shape[0]),
+                (int(detector_shape[0]),
+                 int(detector_shape[1]),
                  self.nw), dtype=np.float32) for ir in range(self.nslice)]
 
         flux_plus_bg_list = [
             np.zeros(
-                (int(detector_shape[1]),
-                 int(detector_shape[0]),
+                (int(detector_shape[0]),
+                 int(detector_shape[1]),
                  self.nw), dtype=np.float32) for ir in range(self.nslice)]
 
+        ### 3. Get background
         if background is not None:
             pupil_thru = self.psf_library.get_pupil_throughput(self.wave[0], instrument_name, aperture_name)
             self.bg = background.mjy_pix * pupil_thru
         else:
             self.bg = self.wave * 0.0
 
-        scene_grid = Grid(psf_pixsize_x, psf_pixsize_y, detector_shape[0] * psf_upsamp, detector_shape[1] * psf_upsamp)
+        ### 4. Create Model Scene Cube(s) from scene and source information
+        self.grid = Grid(psf_pixsize_x * psf_upsamp, psf_pixsize_y * psf_upsamp, detector_shape[1], detector_shape[0])
+        fine_grid = Grid(psf_pixsize_x, psf_pixsize_y, detector_shape[1] * psf_upsamp, detector_shape[0] * psf_upsamp)
         psf_associations = self.psf_library.associate_offset_to_source(self.scene.sources, instrument_name, aperture_name)
         unique_offsets = list(set(psf_associations))
 
         current_scenes = []
         for unique_offset in unique_offsets:
             offset_indices = [i for (i, v) in enumerate(psf_associations) if v == unique_offset]
-            current_scene = ModelSceneCube([self.source_spectra[i] for i in offset_indices], scene_grid)
+            current_scene = ModelSceneCube([self.source_spectra[i] for i in offset_indices], fine_grid)
             # Catch calculations that are too bright to be computed, and have them abort immediately.
             if not np.all(np.isfinite(current_scene.int)):
                 # warn the user about what they've done
@@ -739,6 +785,7 @@ class ConvolvedSceneCube(object):
                                              "correctly. Check your sources and source normalizations.")
             current_scenes.append(current_scene)
 
+        ### 5. Speedup check: Is this a single centered point source?
         # Check whether we have only a single point source near the center
         # (if we do, the convolution can be faster because we don't have to convolve a field larger
         # than the PSF kernel size, even if the scene is formally larger)
@@ -751,20 +798,116 @@ class ConvolvedSceneCube(object):
             elif src.shape['geometry'] != 'point':
                 self.single_point_source = False
 
+        ### 6. Check and prepare scene parameters
+        scene_npix_x = detector_shape[1] * psf_upsamp
+        scene_npix_y = detector_shape[0] * psf_upsamp
+        npix_x = int(detector_shape[1])
+        npix_y = int(detector_shape[0])
+        # Check to make sure npix is an integer
+        if not (scene_npix_x % psf_upsamp) == 0:
+            # This should never happen
+            message = "Number of scene pixels, {}, is not divisible by the PSF upsampling factor, {}".format(
+                scene_npix_x, psf_upsamp)
+            raise DataConfigurationError(value=message)
+        # Check to make sure npix is an integer
+        if not (scene_npix_y % psf_upsamp) == 0:
+            # This should never happen
+            message = "Number of scene pixels, {}, is not divisible by the PSF upsampling factor, {}".format(
+                scene_npix_y, psf_upsamp)
+            raise DataConfigurationError(value=message)
+        # close enough?
+        # TODO: Use numpy.isclose with our usual parameters?
+        if (np.abs(current_scene.xsamp / psf_pixsize_x - 1) > 1e-10):
+            raise ValueError("scene sampling must be the same as PSF sampling")
+        if (np.abs(current_scene.ysamp / psf_pixsize_y - 1) > 1e-10):
+            raise ValueError("scene sampling must be the same as PSF sampling")
+        # We have to make sure npix is an integer or it can't be used as an index
+        new_shape = (npix_y, npix_x)
+
+        ### 7. Make PSF convolution window parameters
+        mini_x = int((scene_npix_x - kernel_npix_x)/2) # minimum index of the kernel size within the FOV
+        maxi_x = int((scene_npix_x + kernel_npix_x)/2) # maximum index of the kernel size within the FOV
+        mini_y = int((scene_npix_y - kernel_npix_y)/2) # minimum index of the kernel size within the FOV
+        maxi_y = int((scene_npix_y + kernel_npix_y)/2) # maximum index of the kernel size within the FOV
+
+        # if we have a rare instance where the PSF is larger than the scene, just effectively convolve it normally
+        # (without this, the tiny window would actually have no pixels in it, producing a zero cube)
+        if kernel_npix_x > scene_npix_x:
+            mini_x = 0
+            maxi_x = scene_npix_x
+        if kernel_npix_y > scene_npix_y:
+            mini_y = 0
+            maxi_y = scene_npix_y
+
+        ### 8. Make the masks
+        fine_masks = []
+        masks = []
+        if self.aper_width is not None and self.aper_height is not None and self.slit_shape=="rectangle":
+            offsets = [(i - (self.nslice - 1) / 2.) * self.aper_width for i in np.arange(self.nslice)]
+            aperture_list = []
+            for offset in offsets:
+                slice_mask_fine = np.zeros((scene_npix_y,scene_npix_x))
+                # Is this a multishutter instrument?
+                if self.multishutter:
+                    for shutter in self.multishutter:
+                        new_mask = fine_grid.rectangular_mask(
+                            width=self.aper_width,
+                            height=self.aper_height,
+                            xoff=offset + shutter[0],
+                            yoff=shutter[1]
+                        )
+                        slice_mask_fine = np.maximum(slice_mask_fine, new_mask)
+                else:
+                    slice_mask_fine = fine_grid.rectangular_mask(
+                        width=self.aper_width,
+                        height=self.aper_height,
+                        xoff=offset,
+                        yoff=0.0
+                    )
+                fine_masks.append(slice_mask_fine)
+                masks.append(AdvancedPSF._rebin(self,slice_mask_fine, new_shape))
+                aperture_list.append({'width': self.aper_width, 'height': self.aper_height, 'offset': (0., offset)})
+        elif self.aper_width is not None and self.slit_shape=="circle":
+            # we do not have a use case for multiple or non-centered circular slits
+            slice_mask_fine = fine_grid.circular_mask(
+                self.aper_width/2, 
+                xoff=0.0, 
+                yoff=0.0
+            )
+            fine_masks.append(slice_mask_fine)
+            masks.append(AdvancedPSF._rebin(self,slice_mask_fine, new_shape))
+            aperture_list = [{'radius': self.aper_width, 'offset': (0., 0)}]
+        else:
+            # The cube indexing order is y,x
+            slice_mask_fine = np.ones((scene_npix_y,scene_npix_x))
+            fine_masks.append(slice_mask_fine)
+            masks.append(AdvancedPSF._rebin(self,slice_mask_fine, new_shape))
+            aperture_list = [self.grid.get_aperture()]
+
+
+        ### 9. Run the convolution for each wavelength, for each unique PSF offset
         for iw in np.arange(self.nw):
             for current_scene, unique_offset, i in zip(current_scenes, unique_offsets, list(range(len(unique_offsets)))):
+                profile = self.psf_library.get_psf(self.wave[iw], instrument_name, aperture_name, source_offset=unique_offset)
+                psf_upsamp = profile['upsamp']
+                kernel = np.asarray(profile['int'], dtype=np.float32)
+
                 if i == 0:
                     psf = AdvancedPSF(
                         self.wave[iw],
-                        self.psf_library,
-                        self.instrument,
-                        psf_source_offset=unique_offset,
-                        model_scene=current_scene,
-                        aper_width=self.aper_width,
-                        aper_height=self.aper_height,
-                        multishutter=self.multishutter,
-                        nslice=self.nslice,
-                        slit_shape=self.slit_shape,
+                        kernel,
+                        psf_upsamp=psf_upsamp,
+                        current_scene=current_scene,
+                        fine_masks=fine_masks,
+                        masks=masks,
+                        fine_shape=(scene_npix_y, scene_npix_x),
+                        new_shape=new_shape,
+                        grid=self.grid,
+                        aperture_list=aperture_list,
+                        mini_x=mini_x,
+                        maxi_x=maxi_x,
+                        mini_y=mini_y,
+                        maxi_y=maxi_y,
                         bg_w=self.bg[iw],
                         single=self.single_point_source,
                         empty_scene = self.empty_scene
@@ -776,15 +919,19 @@ class ConvolvedSceneCube(object):
                     psf.add_intensity(
                         AdvancedPSF(
                             self.wave[iw],
-                            self.psf_library,
-                            self.instrument,
-                            psf_source_offset=unique_offset,
-                            model_scene=current_scene,
-                            aper_width=self.aper_width,
-                            aper_height=self.aper_height,
-                            multishutter=self.multishutter,
-                            nslice=self.nslice,
-                            slit_shape=self.slit_shape,
+                            kernel,
+                            psf_upsamp=psf_upsamp,
+                            current_scene=current_scene,
+                            fine_masks=fine_masks,
+                            masks=masks,
+                            fine_shape=(scene_npix_x, scene_npix_y),
+                            new_shape=new_shape,
+                            grid=self.grid,
+                            aperture_list=aperture_list,
+                            mini_x=mini_x,
+                            maxi_x=maxi_x,
+                            mini_y=mini_y,
+                            maxi_y=maxi_y,
                             bg_w=0.
                         )
                     )
@@ -794,7 +941,7 @@ class ConvolvedSceneCube(object):
                 flux_cube_list[islice][:, :, iw] = psf.slice_int_list[islice]
                 flux_plus_bg_list[islice][:, :, iw] = psf.slice_int_plus_bg_list[islice]
 
-        return psf.grid, psf.aperture_list, flux_cube_list, flux_plus_bg_list
+        return self.grid, aperture_list, flux_cube_list, flux_plus_bg_list
 
     def spectral_model_transform(self):
         """
@@ -898,57 +1045,23 @@ class AdvancedPSF(object):
         If True, assume that we only have a single point source near or at the center, and then
         only convolve a field the size of the PSF kernel (leaving the rest of the FOV at zero source flux).
     """
-
-    def __init__(self, wave, psf_library, instrument, model_scene=None, psf_source_offset=(0, 0),
-                 aper_width=None, aper_height=None, multishutter=[(0.0, 0.0)], nslice=None, slit_shape=None, 
+    def __init__(self, wave, kernel, psf_upsamp=1, current_scene=None, fine_masks=None, masks=None, grid=None,
+                 aperture_list=[], mini_x=0, maxi_x=-1, mini_y=0, maxi_y=-1, fine_shape=(0, 0), new_shape=(0, 0),
                  bg_w=0, single=False, empty_scene=False):
 
+        # For API compatibility
+        self.aperture_list = aperture_list
+        self.grid = grid
         self.warnings = {}
-        instrument_name = instrument.get_name()
-        aperture_name = instrument.get_aperture()
 
-        # The advanced PSF gets its intensity map from a PSF library
-        profile = psf_library.get_psf(wave, instrument_name, aperture_name, source_offset=psf_source_offset)
-        psf_upsamp = profile['upsamp']
-        psf_pixscl_x = profile['pix_sclx']
-        psf_pixscl_y = profile['pix_scly']
+        windex = np.abs(current_scene.wave - wave).argmin() # find nearest index
 
-        #if profile['int'].shape[0] != profile['int'].shape[1]:
-        #    raise ValueError("The PSF must have a square grid shape nx=ny")
+        self.slice_int_list = []
+        self.slice_int_plus_bg_list = []
+        self.slice_mask_list = []
 
-        scene_npix_x = model_scene.nx
-        scene_npix_y = model_scene.ny
-        kernel = np.asarray(profile['int'], dtype=np.float32)
-        npix_x = int(scene_npix_x/psf_upsamp)
-        npix_y = int(scene_npix_y/psf_upsamp)
-        # Check to make sure npix is an integer
-        if not (scene_npix_x % psf_upsamp) == 0:
-            # This should never happen
-            message = "Number of scene pixels, {}, is not divisible by the PSF upsampling factor, {}".format(
-                scene_npix_x, psf_upsamp)
-            raise DataConfigurationError(value=message)
-        # Check to make sure npix is an integer
-        if not (scene_npix_y % psf_upsamp) == 0:
-            # This should never happen
-            message = "Number of scene pixels, {}, is not divisible by the PSF upsampling factor, {}".format(
-                scene_npix_y, psf_upsamp)
-            raise DataConfigurationError(value=message)
-        
-        # We have to make sure npix is an integer or it can't be used as an index
-        new_shape = (npix_x, npix_y)
-
-        # If there is a scene, convolve with it
         if empty_scene:
-            self.intensity = np.zeros((scene_npix_x,scene_npix_y))
-            # self.intensity = profile['int']
-        else:
-            # close enough?
-            if (np.abs(model_scene.xsamp / psf_pixscl_x - 1) > 1e-10):
-                raise ValueError("scene sampling must be the same as PSF sampling")
-            if (np.abs(model_scene.ysamp / psf_pixscl_y - 1) > 1e-10):
-                raise ValueError("scene sampling must be the same as PSF sampling")
-            windex = np.abs(model_scene.wave - wave).argmin() # find nearest index
-
+            self.intensity = np.zeros(fine_shape)
             # See Issue #71 for discussion and benchmarks about
             # these different FFT methods. astropy.convolution plus pyFFTW yields a significant
             # speed-up, of order 30% or more.
@@ -960,30 +1073,14 @@ class AdvancedPSF(object):
             # Basic astropy.convolution with FFTs with parameters set to produce numbers that
             # match the original method
             # self.intensity = convolve_fft(model_scene.int[:, :, windex], kernel[:-1, :-1], normalize_kernel=False)
-            if single:
- 
-                kernel_npix_x, kernel_npix_y = kernel.shape
-                mini_x = int((scene_npix_x - kernel_npix_x)/2) # minimum index of the kernel size within the FOV
-                maxi_x = int((scene_npix_x + kernel_npix_x)/2) # maximum index of the kernel size within the FOV
-                mini_y = int((scene_npix_y - kernel_npix_y)/2) # minimum index of the kernel size within the FOV
-                maxi_y = int((scene_npix_y + kernel_npix_y)/2) # maximum index of the kernel size within the FOV
-
-                # if we have a rare instance where the PSF is larger than the scene, just effectively convolve it normally
-                # (without this, the tiny window would actually have no pixels in it, producing a zero cube)
-                if kernel_npix_x > scene_npix_x:
-                    mini_x = 0
-                    maxi_x = scene_npix_x
-                if kernel_npix_y > scene_npix_y:
-                    mini_y = 0
-                    maxi_y = scene_npix_y
-
-                # This does not use the grid class, so indexing must be y,x (see the Grid class for more)
-                self.intensity = np.zeros((scene_npix_y, scene_npix_x))
-                # mode='same' forces the FFT output to be the same shape as the input scene to be convolved with
-                # (first argument to the function)
-                self.intensity[mini_y:maxi_y, mini_x:maxi_x] = scipy.signal.fftconvolve(model_scene.int[mini_y:maxi_y, mini_x:maxi_x, windex],kernel,mode='same')
-            else:
-                self.intensity = scipy.signal.fftconvolve(model_scene.int[:, :, windex], kernel, mode='same')
+        elif single:
+            # This does not use the grid class, so indexing must be y,x (see the Grid class for more)
+            self.intensity = np.zeros(fine_shape)
+            # mode='same' forces the FFT output to be the same shape as the input scene to be convolved with
+            # (first argument to the function)
+            self.intensity[mini_y:maxi_y, mini_x:maxi_x] = scipy.signal.fftconvolve(current_scene.int[mini_y:maxi_y, mini_x:maxi_x, windex],kernel,mode='same')
+        else:
+            self.intensity = scipy.signal.fftconvolve(current_scene.int[:, :, windex], kernel, mode='same')
 
         # Add the background
         self.intensity_plus_bg = self.intensity + bg_w / psf_upsamp ** 2
@@ -995,67 +1092,15 @@ class AdvancedPSF(object):
         offset rectangles. In principle, one could create an IFU with each slice consisting of discrete shutters.
         This could be used to simulate different IFU designs, such as lenslet or micro-mirror arrays.
         """
-        if aper_width is not None and aper_height is not None and slit_shape=="rectangle":
-            offsets = [(i - (nslice - 1) / 2.) * aper_width for i in np.arange(nslice)]
-            self.slice_int_list = []
-            self.slice_int_plus_bg_list = []
-            self.slice_mask_list = []
-            self.aperture_list = []
-            self.grid = Grid(psf_pixscl_x * psf_upsamp, psf_pixscl_y * psf_upsamp, npix_x, npix_y)
-            fine_grid = Grid(psf_pixscl_x, psf_pixscl_y, scene_npix_x, scene_npix_y)
-            for offset in offsets:
-                slice_mask_fine = np.zeros(self.intensity.shape)
-                # Is this a multishutter instrument?
-                if multishutter:
-                    for shutter in multishutter:
-                        new_mask = fine_grid.rectangular_mask(
-                            width=aper_width,
-                            height=aper_height,
-                            xoff=offset + shutter[0],
-                            yoff=shutter[1]
-                        )
-                        slice_mask_fine = np.maximum(slice_mask_fine, new_mask)
-                else:
-                    slice_mask_fine = fine_grid.rectangular_mask(
-                        width=aper_width,
-                        height=aper_height,
-                        xoff=offset,
-                        yoff=0.0
-                    )
+        for idx, fine_mask in enumerate(fine_masks):
+            slice_int, slice_int_plus_bg = self._apply_slit_mask(fine_mask, new_shape)
+            # for this purpose a set of multiple shutters is treated as a single aperture and uses
+            # the properties of the central shutter.
+            
+            self.slice_int_list.append(slice_int)
+            self.slice_int_plus_bg_list.append(slice_int_plus_bg)
+            self.slice_mask_list.append(masks[idx])
 
-                slice_int, slice_int_plus_bg, slice_mask = self._apply_slit_mask(slice_mask_fine, new_shape)
-                # for this purpose a set of multiple shutters is treated as a single aperture and uses
-                # the properties of the central shutter.
-                aperture = {'width': aper_width, 'height': aper_height, 'offset': (0., offset)}
-                self.slice_int_list.append(slice_int)
-                self.slice_int_plus_bg_list.append(slice_int_plus_bg)
-                self.slice_mask_list.append(slice_mask)
-                self.aperture_list.append(aperture)
-        elif aper_width is not None and slit_shape=="circle":
-            self.grid = Grid(psf_pixscl_x * psf_upsamp, psf_pixscl_y * psf_upsamp, npix_x, npix_y)
-            fine_grid = Grid(psf_pixscl_x, psf_pixscl_y, scene_npix_x, scene_npix_y)
-            slice_mask_fine = np.zeros(self.intensity.shape)
-            # we do not have a use case for multiple or non-centered circular slits
-            slice_mask_fine = fine_grid.circular_mask(
-                aper_width/2, 
-                xoff=0.0, 
-                yoff=0.0
-            )
-            slice_int, slice_int_plus_bg, slice_mask = self._apply_slit_mask(slice_mask_fine, new_shape)
-            aperture = {'radius': aper_width, 'offset': (0., 0)}
-            self.slice_int_list = [slice_int]
-            self.slice_int_plus_bg_list = [slice_int_plus_bg]
-            self.slice_mask_list = [slice_mask]
-            self.aperture_list = [aperture]
-        else:
-            # The cube indexing order is y,x
-            slice_mask_fine = np.ones((scene_npix_y, scene_npix_x))
-            slice_int, slice_int_plus_bg, slice_mask = self._apply_slit_mask(slice_mask_fine, new_shape)
-            self.grid = Grid(psf_pixscl_x * psf_upsamp, psf_pixscl_y * psf_upsamp, npix_x, npix_y)
-            self.slice_int_list = [slice_int]
-            self.slice_int_plus_bg_list = [slice_int_plus_bg]
-            self.slice_mask_list = [slice_mask]
-            self.aperture_list = [self.grid.get_aperture()]
 
     def add_intensity(self, psf):
         """
@@ -1091,8 +1136,7 @@ class AdvancedPSF(object):
         """
         intensity = self._rebin(self.intensity * slit_mask, new_shape)
         intensity_plus_bg = self._rebin(self.intensity_plus_bg * slit_mask, new_shape)
-        aperture_mask = self._rebin(slit_mask, new_shape)
-        return intensity, intensity_plus_bg, aperture_mask
+        return intensity, intensity_plus_bg
 
     def _rebin(self, a, shape):
         """
@@ -1111,7 +1155,7 @@ class AdvancedPSF(object):
         """
         # While the array shape is defined as x,y, it's actually implemented as y,x; hence the subscripts
         # see coord.py Grid definition for more explanation
-        sh = int(shape[1]), int(a.shape[0] // shape[1]), int(shape[0]), int(a.shape[1] // shape[0])
+        sh = int(shape[0]), int(a.shape[0] // shape[0]), int(shape[1]), int(a.shape[1] // shape[1])
         new = a.reshape(sh).sum(-1).sum(1)
         return new
 
