@@ -69,6 +69,8 @@ class Background(cf.DefaultConfig):
         self.telescope = observation.instrument.telescope.tel_name
 
         DefaultConfig.__init__(self, config=config, webapp=webapp, **kwargs)
+        self.bg_location_default = self.bg_location
+        self.bg_level_default = self.bg_level
 
         self.ref_dir = os.path.join(default_refdata_directory, self.telescope, 'background')
         self.bg_location = observation.background
@@ -136,8 +138,9 @@ class Background(cf.DefaultConfig):
         """
 
         if self.bg_location == 'none' or self.bg_level == 'none':
-            bg_wave = np.array([0.01, 30.0]) << PANDEIA_WAVEUNITS
-            bg_sb = np.array([0.0, 0.0]) << PANDEIA_FLUXUNITS
+            bg_file = os.path.join(default_refdata_directory, self.telescope, "background", "{0}_{1}.fits".format(self.bg_location_default,self.bg_level_default))
+            bg_wave = ref_data_column(bg_file, colname='wavelength') << PANDEIA_WAVEUNITS
+            bg_sb = ref_data_column(bg_file, colname='background') * 0 << u.MJy #/u.sr, but that's not valid in spectra
             try:
                 # Using ConstFlux1D would work, but then the SourceSpectrum has no waveset, and the rest of the functions expect one.
                 tot_bg = syn.spectrum.SourceSpectrum(Empirical1D, points=bg_wave, lookup_table=bg_sb)
@@ -186,7 +189,7 @@ class Background(cf.DefaultConfig):
         """
         i_nu = self.bg(self.bg.waveset, flux_unit=PANDEIA_FLUXUNITS).to_value(PANDEIA_FLUXUNITS) * (u.arcsec * u.arcsec).to(u.sr)
 
-        i_nu_pix = i_nu * self.aperture_pars['pix'] ** 2
+        i_nu_pix = i_nu * self.aperture_pars['plate_scale'][0] * self.aperture_pars['plate_scale'][1]
         return i_nu_pix
 
     def trim(self, wmin, wmax):
@@ -250,5 +253,5 @@ class Background(cf.DefaultConfig):
             Flux per pixel in photons/s/cm^2/pixel
         """
         i_lambda = 1.50916e9 / self.wave * self.bg(self.bg.waveset, flux_unit=syn.units.FLAM).value << (u.arcsec * u.arcsec).to(u.sr)
-        i_lambda_pix = i_lambda * self.aperture_pars['pix'] ** 2
+        i_lambda_pix = i_lambda * self.aperture_pars['plate_scale'][0] * self.aperture_pars['plate_scale'][1]
         return i_lambda_pix
