@@ -2,25 +2,21 @@
 from __future__ import division, print_function, absolute_import
 
 import os
+import json
 import pkg_resources
+
 import numpy as np
 
 from . import config
 from .io_utils import read_json
-from pandeia.engine.source import Source, Line
+from pandeia.engine.source import Source, Line, load_empty_source
 from pandeia.engine.custom_exceptions import EngineInputError
 from pandeia.engine.normalization import NormalizationFactory
 from pandeia.engine.sed import SEDFactory
+from pandeia.engine.utils import apply_default_only
 
 default_refdata_directory = config.default_refdata_directory
 
-# Python 2 triggers IOError instead of FileNotFoundError.
-try:
-    FileNotFoundError
-except NameError:
-    FileNotFoundError = OSError
-
-
 def strip_meta(d):
     """
     recursively strip any 'meta' tags from a dict, d. this function modifies the dict in place.
@@ -76,7 +72,7 @@ def get_instrument_config(telescope, instrument):
     return config
 
 
-def build_default_source(geometry="point"):
+def build_default_source(telescope, geometry="point"):
     """
     Build and return a default Source in engine API dict format
 
@@ -85,7 +81,8 @@ def build_default_source(geometry="point"):
     src: dict
         Source configuration data in engine API format
     """
-    src = Source(shape={"geometry": geometry}).as_dict()
+    src = Source(telescope, shape={"geometry": geometry}).as_dict()
+
     src['spectrum']['normalization'] = NormalizationFactory(config=src['spectrum']['normalization']).as_dict()
     src['spectrum']['sed'] = SEDFactory(config=src['spectrum']['sed']).as_dict()
     return src
@@ -104,7 +101,7 @@ def build_default_line():
     return line
 
 
-def build_default_scene():
+def build_default_scene(telescope):
     """
     Build and return a default scene which consists of a single default Source
 
@@ -113,27 +110,36 @@ def build_default_scene():
     scene: list
         Single element list containing a source as built by build_default_source()
     """
-    scene = [build_default_source()]
+    scene = [build_default_source(telescope)]
     return scene
 
 
 def build_empty_scene():
     """
-    Build and return an empty scene. Because of the way ConvolvedSceneCube's and AstroSpectrum's are created,
-    there must be some concept of a spectrum and thus a source contained within a scene. The (admittedly hacky)
-    workaround is to build a default scene with a flat-spectrum point source and set its flux to 0. This will provide
-    the framework necessary to build things from this scene using the existing code, but not add any actual flux. The
-    primary motivation for this is the BNC which needs to build a ConvolvedSceneCube that only contains background signal.
+    Build and return an empty scene. Because of the way ConvolvedSceneCubes and AstroSpectrums are created,
+    there must be some concept of a spectrum and thus a source contained within a scene. The (admittedly
+    hacky) workaround is to build a default scene with a flat-spectrum point source and set its flux to 0.
+    This will provide the framework necessary to build things from this scene using the existing code, but not
+    add any actual flux. The primary motivation for this is the BNC which needs to build a ConvolvedSceneCube
+    that only contains background signal.
+
+    As for the scene itself, we have to define a complete source here; the idea of an empty scene depending on
+    the telescope is perhaps a bit too much, and we need to define the complete normalization here anyway (for
+    some units, e.g. Vegamags, 0 is bright, not faint.)
+
+    As of 2022.0606, this definition is equivalent to the default JWST/Roman source with flux set to zero. It
+    must be kept up-to-date with our source API.
 
     Returns
     -------
     scene: list
         Single element list containing a zero-flux source
     """
-    scene = build_default_scene()
-    scene[0]['spectrum']['normalization']['norm_flux'] = 0.0
-    return scene
-
+    src = load_empty_source()
+    src['spectrum']['normalization'] = NormalizationFactory(config=src['spectrum']['normalization']).as_dict()
+    src['spectrum']['sed'] = SEDFactory(config=src['spectrum']['sed']).as_dict()
+    src['spectrum']['normalization']['norm_flux'] = 0.0
+    return [src]
 
 def build_default_calc(telescope, instrument, mode, **kwargs):
     """
@@ -163,7 +169,7 @@ def build_default_calc(telescope, instrument, mode, **kwargs):
     # the instrument configuration now contains instrument and strategy values
     calc['configuration'] = ins_config['defaults'][mode]['instrument_args']
 
-    calc['scene'] = build_default_scene()
+    calc['scene'] = build_default_scene(telescope)
 
     bg_defaults_file = "defaults/background_defaults.json"
     bg_defaults = read_json(pkg_resources.resource_filename("pandeia.engine", bg_defaults_file))
@@ -232,6 +238,8 @@ def build_default_calc(telescope, instrument, mode, **kwargs):
                 key = calc['configuration']['instrument'][sky_key]
                 calc['strategy'][params[p]] = st_defaults[p][key]
 
+    calc = apply_default_only(calc)
+
     return calc
 
 
@@ -299,7 +307,7 @@ def calcs_to_wb(calc_list, name="Generated workbook", note="", desc="", wb_id=1,
         calcs[str(calc_i)]["insmode"] = calcs[str(calc_i)]["camera_config"]["instrument"].pop("mode")
         calc_i += 1
     # now populate the workbook
-    wb["test_mode"] = "1"
+    wb["test_mode"] = "0"
     wb["created"] = 0
     wb["deleted"] = 0
     wb["id"] = wb_id
