@@ -67,3 +67,6 @@ DEFAULT_DISPERSION = 0.01
 
 # Minimum to clip values to, for avoidance of divsion-by-zero errors in exposure timing, profile generation, etc.
 MIN_CLIP = 1e-10
+
+# How many parts to split the extraction aperture into (for the TASpec strategy)
+EXTRACTION_NSPLIT=1000
