@@ -294,7 +294,7 @@ class Grid(object):
         if width is None:
             width = maxx - minx + 2 * self.xsamp
         if height is None:
-            height = maxy - miny + 2 * self.xsamp
+            height = maxy - miny + 2 * self.ysamp
 
         mask_x = np.array(
             [
