@@ -1,12 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import division, absolute_import
 
+import bisect
+import warnings
+
 import numpy as np
 import numpy.ma as ma
+from astropy.modeling.functional_models import Box1D
 from .custom_exceptions import EngineInputError, DataConfigurationError
 from .constants import MIN_CLIP
 from .exposure_factory import ExposureFactory
-
+from .utils import spectrum_resample
 class Detector:
 
     def __init__(self, config, webapp=False):
@@ -52,11 +56,25 @@ class Detector:
 
         return exposure_spec
 
-    def set_saturation(self, toggle):
-        # this function turns saturation on and off - if toggle is true, we 
-        # compute saturation. If toggle is false, we do not.
-        if not toggle:
-            self.fullwell = 1.0e99
+    def set_saturation(self, flag):
+        # this function turns saturation on and off - if the flag is True, we 
+        # compute saturation. If toggle is False, we do not.
+        if not flag:
+            self.fullwell = np.inf
+
+    def set_readnoise(self, flag):
+        # this function turns readnoise on and off - if the flag is False,
+        # we set the readnoise to 0.
+        if not flag:
+            self.rn = 0
+
+    def set_excess(self, flag):
+        # this function turns the various excess noise components on and off - if the flag is False,
+        # we set var_fudge, excessp1, and excessp2 to 0.
+        if not flag:
+            self.var_fudge = 1
+            self.excessp1 = 0
+            self.excessp2 = 0
 
     def get_slope_variance(self, rate, unsat):
         # This function allows different slope variance functions (used for MULTIACCUM TA)
@@ -69,6 +87,96 @@ class Detector:
     def set_time(self, new_time):
         raise NotImplementedError("Reverse ETC calculations for {} are not implemented.".format(type(self)))
 
+    def to_resels_scalar(self, wave, values, refwave):
+        """
+        Extract scalar value from array in resels.
+        Requires an array, a resel (read from the detector), and the reference wavelength
+        Uses astropy 1D box model to apply the resel value.
+        The one advance this has over previous Pandeia behavior is that you DO get the (value) at the
+        average position of your reference wavelength.
+
+        This works in any case where you're dropping the wavelength dimension, so also cube->2D
+
+        Parameters
+        ----------
+        wave: ndarray
+            the wavelength array, in microns
+        values: ndarray
+            the value array 
+        refwave: float
+            the reference wavelength in microns
+
+        Returns
+        -------
+        value: float
+            the value in resels
+        """
+        values = ma.fix_invalid(values, fill_value=0.0).data
+        if len(wave) > 1:
+            if hasattr(self,"pixels_per_resel"):
+                # find out what fractional index position the reference wavelength is at
+                cenidx = np.interp(refwave, wave, np.arange(len(wave)))
+                mask_pix = np.array([0.0, 0.0, 1.0, 1.0, 0.0, 0.0])
+                mask = np.array(
+                    [
+                        0,
+                        cenidx - (self.pixels_per_resel + 1) / 2.0,
+                        cenidx - (self.pixels_per_resel - 1) / 2.0,
+                        cenidx + (self.pixels_per_resel - 1) / 2.0,
+                        cenidx + (self.pixels_per_resel + 1) / 2.0,
+                        len(wave)-1
+                    ]
+                )
+                boxfilter = np.interp(np.arange(len(wave)), mask, mask_pix)
+                if len(values.shape) == 3: # if we're doing a cube, the boxfilter needs to be reshaped
+                    boxfilter = boxfilter.reshape(-1,1,1)
+                resel_value = np.sum(values*boxfilter)/self.pixels_per_resel # the 1D values are simply extracted over the pixel-area of one resel
+            else:
+                # there are no resels but it is a 1D array
+                wave_index = (np.abs(wave - refwave)).argmin()
+                resel_value = values[wave_index]
+        else:
+            # it is a single-element array
+            resel_value = values[0]
+
+        return resel_value
+
+    # TODO: Resels for downsampling a cube
+
+    def to_resels_array(self, wave, values):
+        """
+        Convert array to resels.
+        Requires an array and a resel (read from the instrument)
+
+        Parameters
+        ----------
+        wave: ndarray
+            the wavelength array, in microns
+        values: ndarray
+            the value array 
+
+        Returns
+        -------
+        rebinwave: ndarray
+            the rebinned wavelength array, in microns
+        array: ndarray
+            the rebinned array
+        """
+        if hasattr(self,"pixels_per_resel"):
+            if self.pixels_per_resel != 1 and len(wave) > 1:
+                # spectrum_resample cannot handle NaNs, so we need to save the nanmask and set nans to 0.
+                nanmask = np.isfinite(values) 
+                values = ma.fix_invalid(values, fill_value=0.0).data
+
+                rebinwave = wave[::self.pixels_per_resel] # this is the same operation conducted by PyETC
+                resel_array = spectrum_resample(values, wave, rebinwave)
+
+                # rebin the nanmask (all elements in resel must be true= not NaN for the resel to not be NaN) and reapply
+                rebinmask = [np.all(nanmask[x:x+self.pixels_per_resel]) for x in range(0, len(values), self.pixels_per_resel)]
+                return rebinwave, resel_array*rebinmask
+        # if none of the above were true, return the originals
+        return wave, values
+
 class Detector_MultiAccum(Detector):
 
     def get_unsaturated(self, slope, full_saturation=2):
@@ -321,7 +429,7 @@ class Detector_MultiAccum(Detector):
 
         Inputs
         ------
-        readnoise: float
+        rn: float
           Readnoise per pixel.
 
         unsat_ngroups: ndarray
@@ -331,9 +439,6 @@ class Detector_MultiAccum(Detector):
            negligible for pixels with signal rates high enough to saturate in part of the ramp.
            This is probably always a good approximation.
 
-        readnoise: float
-          Readnoise per pixel.
-
         Returns
         -------
         var_rn: ndarray if unsat_ngroups is supplied, float if unsat_ngroups is not supplied.
@@ -366,7 +471,7 @@ class Detector_MultiAccum(Detector):
             var_rn += excess_variance
         return var_rn
 
-    def _get_saturation_mask(self, rate, saturation_mask):
+    def _get_saturation_mask(self, rate):
         """
         Compute a numpy array indicating pixels with full saturation (2), partial saturation (1) and no saturation (0).
 
@@ -380,6 +485,7 @@ class Detector_MultiAccum(Detector):
         mask: 2D np.ndarray
             Saturation mask image
         """
+        saturation_mask = np.zeros(rate.shape)
         unsat_ngroups = self.get_unsaturated(rate, self.fullwell)
 
         saturation_mask[(unsat_ngroups < self.exposure_spec.ngroup)] = 1
@@ -484,14 +590,43 @@ class SiAs(Detector_MultiAccum):
 
 class H4RG(H2RG):
     """
-    Detector properties for H4RG detectors (Roman WFI). Currently no different from H2RGs.
+    Detector properties for H4RG detectors (Roman WFI).
     """
-    pass
+    def set_excess(self, flag):
+        # this function turns excess noise on and off. H4RG detectors do not use excess noise parameters. Warn
+        # if a user tries to turn it on.
+        if flag:
+            warnings.warn(f"{type(self).__name__} detectors do not have excess noise parameters, they cannot be turned on.")
 
 #------------ Single accum ------------------
 
 class Detector_SingleAccum(Detector):
 
+    def set_excess(self, flag):
+        # this function turns excess noise on and off. SingleAccum detectors do not use H2RG-style excess
+        # noise parameters. Warn if a user tries to turn them on.
+        if flag:
+            warnings.warn(f"{type(self).__name__} detectors do not have excess noise parameters, they cannot be turned on.")
+
+    def postflash_as_rate(self):
+        """
+        Postflash will not always be defined (and is only valid for some CCDs)
+
+        Postflash must be a positive integer number of electrons.
+
+        It's not really a rate, but to add it to the target flux we must treat its contribution as one, which
+        means dividing it by the single exposure time - the flash happens before every independent exposure.
+        """
+        if (("postflash" in self.input_detector) and not self.__dict__.get("has_postflash", False)):
+            raise EngineInputError(f"Detector does not support postflash.")
+        postflash = self.input_detector.get("postflash", 0)
+        if (not isinstance(postflash, int)) or (postflash < 0):
+            raise EngineInputError(f"Postflash value {postflash} invalid. Must be positive integer.")
+
+        # now convert to rate.
+        postflash_rate = postflash / self.exposure_spec.exposure_time
+        return postflash_rate
+
     def get_before_sat(self, rate):
         """
         This method returns a map of the maximum time each pixel can be exposed before saturation.
@@ -529,7 +664,7 @@ class Detector_SingleAccum(Detector):
             slope_array = slope.clip(MIN_CLIP, np.max([MIN_CLIP, np.max(slope)]))
             return self.exposure_spec.saturation_time / (self.fullwell / slope_array)
 
-    def _get_saturation_mask(self, rate, saturation_mask):
+    def _get_saturation_mask(self, rate):
         """
         Compute a numpy array indicating pixels with (full) saturation (2), and no saturation (0).
 
@@ -543,6 +678,7 @@ class Detector_SingleAccum(Detector):
         mask: 2D np.ndarray
             Saturation mask image
         """
+        saturation_mask = np.zeros(rate.shape)
 
         saturation_fraction = self.get_saturation_fraction(rate)
 
@@ -575,17 +711,19 @@ class Detector_SingleAccum(Detector):
             The associated variance of the readnoise only
         """
 
-        # The noise calculation depends on the pixel rate BEFORE IPC convolution. fp_pix_variance takes that pre-IPC
-        # rate and scales it by the quantum yield and Fano factor to get the per-pixel variance in the electron rate.
+        # The noise calculation depends on the pixel rate BEFORE IPC convolution. fp_pix_variance takes that
+        # pre-IPC rate and scales it by the quantum yield and Fano factor to get the per-pixel variance in the
+        # electron rate.
         
-        # TODO: Determine if further changes to this equation need to be made. (JETC-2121)
-        # The reverse calculation equation, derived from this noise computation, does not include Fano noise, nor does
-        # the PyETC version. If we use a signal that includes fano noise, the reverse calculation does not reverse the 
-        # SNR calculation by a factor of more than 20%. Without it, the calculation is discrepant by ~1%.
-        # More to the point, the Fano noise trends negative at longer wavelengths, which is almost certainly unphysical.
-        # PyETC does not include Fano noise even though it's supposed to apply to detector effects.
-        # It may yet be necessary to alter the Fano factor (set to 0) and quantum yield calculations so that we CAN use 
-        # an fp_pix_variance property that is never run through IPC convolution.
+        # TODO: Determine if further changes to this equation need to be made. (JETC-2121) The reverse
+        # calculation equation, derived from this noise computation, does not include Fano noise, nor does the
+        # PyETC version. If we use a signal that includes fano noise, the reverse calculation does not reverse
+        # the SNR calculation by a factor of more than 20%. Without it, the calculation is discrepant by ~1%.
+        # More to the point, the Fano noise trends negative at longer wavelengths, which is almost certainly
+        # unphysical. PyETC does not include Fano noise even though it's supposed to apply to detector
+        # effects. It may yet be necessary to alter the Fano factor (set to 0) and quantum yield calculations
+        # so that we CAN use an fp_pix_variance property that is never run through IPC convolution.
+
         variance_per_pix = rate['fp_pix'] / self.exposure_spec.measurement_time
 
         # SingleAccum detectors are read once, but signal variance and dark current are time-dependent.
@@ -594,10 +732,10 @@ class Detector_SingleAccum(Detector):
 
         # Noise is only valid when the pixel isn't saturated - on MultiAccum, that means operating on just the
         # unsaturated groups. In SingleAccum it means masking out the saturated pixels - they have no noise.
-        # The default fill value for masked arrays is a finite number, so convert to ndarrays, and fill with NaNs
-        # to make sure missing values are interpreted as truly undefined downstream.
+        # The default fill value for masked arrays is a finite number, so convert to ndarrays, and fill with
+        # NaNs to make sure missing values are interpreted as truly undefined downstream.
 
-        mask = self._get_saturation_mask(rate, np.ones_like(variance_per_pix))
+        mask = self._get_saturation_mask(rate['fp_pix'])
 
         slope_var[mask==2] = np.nan
         slope_rn_var[mask==2] = np.nan
@@ -664,6 +802,26 @@ class Detector_SingleAccum(Detector):
 
         return unsat_map
 
+    def compute_scattering_factor(self, wave):
+        """
+        Compute the echelle scattered light parameter.
+
+        Parameters
+        ----------
+        wave: float, numpy.ndarray
+            Wavelength value(s) for which the scattering is desired.
+
+        Returns
+        -------
+        scatter_factor
+
+        """
+        wave_scatter, mult_scatter = self.echelle_scattering
+        scattering_val = np.interp(wave, wave_scatter, mult_scatter)
+        scatter_factor = (1. / scattering_val - 1.)
+
+        return scatter_factor
+
     def _get_variance_fudge(self, wave):
         """
         Some instruments (e.g. MIRI) have measured noise properties that are not well explained
@@ -688,28 +846,136 @@ class Detector_SingleAccum(Detector):
         self.exposure_spec.total_exposure_time = new_time
 
     def scale_exposure_time(self, extracted, input):
+        """
+        Scale the exposure time to match the SNR we want.
+
+        To do this requires a FULL understanding of the SingleAccum noise calculation done in the 
+        Strategy and Report classes, because we need to implement its reverse here.
+
+        This is only valid for SingleAccum calculations, where the function is:
+        SNR = SR / sqrt(SR/T + BR/T + DR/T + ((Rn**2 + P) * nreads * npix)/T**2)
+        and
+        SR = Source Rate (electrons/s)
+        BR = Background Rate (electrons/s)
+        DR = Dark Current Rate (electrons/s)
+        Rn = Readnoise (electrons)
+        P = Postflash (electrons)
+        nreads = Number of independent reads
+        npix = Pixels over which Readnoise and Postflash are given (as they are per-pixel 
+         values still)
+        T = Time (s)
+        (note the differences between this and the pyetc time formula - pyetc deals in counts; 
+        Pandeia deals with rates)
+
+        Per JETC-2065, solving for T (with an expected input SNR) yields this equation:
+        0 = (Rn**2 + P)*npix*nreads + (SR + BR + DR) * T - (SR/SNR)**2 * T**2
+        Which is a quadratic equation, solveable in the usual way.
+
+        The complications to this (JETC-2121) are as follows:
+        1. The Strategy outputs the extracted flux rate (SR) composed of just the source rate 
+           itself, and a flux_plus_bg rate composed of the source, background, dark current, WFC3 
+           thermal background, and postflash (SBDPR) - everything but readnoise, which only ever
+           appears as a noise term. 
+           - Postflash is included as a rate in SBDPR so that it appears in the 2D signal 
+             products, but it is fundamentally not a rate and must be removed from SBDPR 
+             (making SBDR) so it can be re-added correctly.
+           - The other components (all of SBDR) are genuine rates.
+           - The noise may include an ADDITIONAL term of STIS Echelle scattering added AFTER 
+             Strategy.extract(), which will need to be accounted for. It is also a rate.
+        2. The correlated noise extraction in the Strategy extracts the signal and noise through 
+           (effectively) different apertures. If M is the mask array, the signal extraction is 
+           M * SR and and the noise extraction is M * NR * M^T (see Strategy._error_sum), which 
+           is equivalent to extracting the noise through M**2, which is smaller than M.
+           Example:
+           If this is the mask: 0    0.5  0
+                                0.5  1    0.5
+                                0    0.5  0       It covers 3 pixels (0.5 + 0.5 + 1 + 0.5 + 0.5)
+           Then the noise aperture is the above matrix squared, or this:
+                                0    0.25 0
+                                0.25  1    0.25
+                                0    0.25 0      It covers 2 pixels (0.25 + 0.25 + 1 + 0.25 + 0.25)
+           This code computes the required noise aperture size (npix_noise) and mask correction 
+           factors (mask_correction), which would be 0.66667 in my simple example here.
+           - Signal is NOT spatially uniform (we cannot guess what the source flux present in those
+             edge pixels is), so we really do need to use an actual extraction through the noise 
+             aperture (extracted['extracted_noise_source']). We also have to remove the extracted 
+             signal from the extracted flux_plus_bg_rate. (SBDR-SR = BDR)
+           - Background, dark current, and thermal are all spatially uniform, and thus can be 
+             handled by multiplying BDR by the mask_correction factor.
+           - Postflash and Readnoise are also spatially uniform and given per pixel, and need to 
+             be multiplied by the number of pixels in the noise aperture.
+           - We do, however, still need the real extracted signal rate for the SR/SNR term.
+
+        Parameters
+        ----------
+        extracted: dict
+            Dictionary of output products produced by Strategy.extract
+        input: dict
+            Dictionary of user inputs
+
+        Returns
+        -------
+        time: float
+            The exposure time to scale to.
+
+        """
         wave_index = 0
-        if len(extracted['wavelength']) > 1:
-            wave_pix = extracted['wavelength']
-            wave_index = int(len(wave_pix) / 2.0)
-            if 'reference_wavelength' in input['strategy']:
-                wref = input['strategy']['reference_wavelength']
-                if wref is not None:
-                    if wref >= wave_pix.min() and wref <= wave_pix.max():
-                        wave_index = (np.abs(wave_pix - wref)).argmin()
-        # the Report version of this conditional produces a warning here (bad_waveref), but we won't 
-        # bother now and let the Report produce it later.
+        wave_pix = extracted['wavelength'] # at worst, this will be backwards from the actual wavelength grid
+                                           # but we explicitly use min and max, so this should be fine.
+        if len(wave_pix) > 1:
+            wref = (wave_pix.max()+wave_pix.min())/2.
+            user_wave = input['strategy']['reference_wavelength']
+            if user_wave is not None:
+                if user_wave >= wave_pix.min() or user_wave <= wave_pix.max():
+                    wref = user_wave
+                # the Report version of this conditional produces a warning here (bad_waveref), but we won't 
+                # bother now and let the Report produce it later.
 
-        sbdr = extracted['extracted_flux_plus_bg'][wave_index] # (source + background + dark) rate
-        sr =  extracted['extracted_flux'][wave_index] # source rate
+            sbdpr = self.to_resels_scalar(wave_pix, extracted['extracted_flux_plus_bg'], wref) # (source + background + dark + postflash) rate
+            sr = self.to_resels_scalar(wave_pix, extracted['extracted_flux'], wref) # source rate
+            ns = self.to_resels_scalar(wave_pix, extracted['extracted_noise_source'], wref)
+            if wref >= wave_pix[-2]:
+                mask = extracted['masks_products'][-1]
+            elif wref <= wave_pix[1]:
+                mask = extracted['masks_products'][0]
+            else:
+                idx = bisect.bisect(wave_pix, wref)
+                mask_0 = extracted['masks_products'][idx]
+                mask_1 = extracted['masks_products'][idx+1]
+                mask = mask_0 + (mask_1 - mask_0) * (wref - wave_pix[idx-1]) / (wave_pix[idx] - wave_pix[idx-1])
+                
+            # Echelle scattering, where it exists, must be accounted for in our noise formula
+            if hasattr(self, "echelle_scattering"):
+                scatter = self.compute_scattering_factor(wref)
+            else:
+                scatter = 0
+        else:
+            sbdpr = extracted['extracted_flux_plus_bg'][0] # (source + background + dark + postflash) rate
+            sr = extracted['extracted_flux'][0] # source rate
+            ns = extracted['extracted_noise_source'][0] # source rate through noise aperture
+            mask = extracted['masks_products'][0]
+            scatter = 0
+            
+        # The SNR we're shooting for, NOT the one we achieved
         snr = input['configuration']['detector']['snr']
-        npix = extracted['extraction_area']
 
-        # We will eventually need to implement postflash here
-        # delta = (sbdr)**2 + 4 * (sr / snr)**2 * (self.postflash + self.readnoise**2) * npix * self.exposure_spec.nsplit
-        delta = (sbdr)**2 + 4 * (sr / snr)**2 * (self.rn**2) * npix * self.exposure_spec.nsplit
+        #snr = sr/noise
 
-        time = (sbdr + np.sqrt(delta)) / (2 * (sr / snr)**2)
+        npix = np.sum(mask)
+        # the noise is computed over effectively fewer pixels, due to the fact that the noise is squared and 
+        # the edges of the mask are only partially transparent.
+        npix_noise = np.sum(mask**2)
+        mask_correction = npix_noise/npix
+
+        # postflash is in the plus_background rate array but it is fundamentally not a time-dependent term.
+        sbdr = sbdpr - self.postflash_as_rate()*npix
+        postflash = self.input_detector.get("postflash", 0)
+        # we also need to remove the signal rate, because it is NOT a uniform term.
+        bdr = (sbdr - sr) * mask_correction
+
+        delta = (ns + bdr + (ns*scatter))**2 + 4 * (sr / snr)**2 * (self.rn**2 + postflash) * npix_noise * self.exposure_spec.nsplit
+
+        time = ((ns + bdr + (ns* scatter)) + np.sqrt(delta)) / (2 * (sr / snr)**2)
 
         return time
 
@@ -735,6 +1001,12 @@ class MAMA(Detector_SingleAccum):
         # MAMA detector has no concept of saturation, so this is irrelevant.
         pass
 
+    def set_readnoise(self, flag):
+        # this function turns readnoise on and off, but MAMAs do not have readnoise to turn on. Warn if a user
+        # tries to turn it on.
+        if flag:
+            warnings.warn(f"{type(self).__name__} detectors do not have readnoise, it cannot be turned on.")
+
     def get_saturation_fraction(self, rate):
         """
         This method returns a map of the fraction of pixels that exceed
