@@ -17,7 +17,6 @@ from . import debug_utils
 from .signal import DetectorSignal, CombinedSignal, CalculationConfig
 from .noise import DetectorNoise
 
-
 # pyfftw.interfaces.cache.enable()
 
 def setup(input, webapp=False):
@@ -50,13 +49,13 @@ def setup(input, webapp=False):
     This section currently implements the Pandeia engine API.  As the engine's object model
     is refactored, this section will have to change accordingly.
     """
+    instrument = InstrumentFactory(config=instrument_configuration, webapp=webapp)
+    warnings.update(instrument.warnings)
+
     # check for empty scene configuration and set it up properly if it is empty.
     if len(scene_configuration) == 0:
         scene_configuration = build_empty_scene()
 
-    instrument = InstrumentFactory(config=instrument_configuration, webapp=webapp)
-    warnings.update(instrument.warnings)
-
     strategy = StrategyFactory(instrument, config=strategy_configuration, webapp=webapp)
 
     return calc_config, instrument, strategy, scene_configuration, background, background_level, warnings
@@ -119,7 +118,7 @@ def make_observation(calc_config, instrument, strategy, scene, background, backg
             # Then loop through a list of orders for which we only want to add the background calculation
             if background_orders is not None:
                 o_empty = copy.deepcopy(obs)
-                o_empty.scene = Scene(input=build_empty_scene(), webapp=webapp)
+                o_empty.scene = Scene(instrument.telescope.tel_name, input=build_empty_scene(), webapp=webapp)
                 for order in background_orders:
                     order_signals.append(
                         DetectorSignal(o_empty, calc_config=calc_config, webapp=webapp, order=order,
@@ -161,7 +160,7 @@ def configure_coronagraphy(input, strategy, scene_configuration, warnings, webap
 
     # add the psf_reference to the scene
     scene_configuration.append(psf_subtraction_configuration)
-    scene = Scene(input=scene_configuration, webapp=webapp)
+    scene = Scene(strategy.instrument.telescope.tel_name, input=scene_configuration, webapp=webapp)
     if hasattr(strategy, "scene_rotation"):
         scene.sources = strategy.rotate(scene.sources)
     warnings.update(scene.warnings)
@@ -198,6 +197,7 @@ def calculate_sn(input, webapp=False):
     -------
     report.Report instance
     """
+
     calc_config, instrument, strategy, scene_configuration, background, background_level, warnings = setup(input, webapp=webapp)
 
     # strategies can have different figures of interest that need to be calculated.
@@ -215,11 +215,17 @@ def calculate_sn(input, webapp=False):
     if contrast:
         scene, strategy = configure_coronagraphy(input, strategy, scene_configuration, warnings, webapp=webapp)
     else:
-        scene = Scene(input=scene_configuration, webapp=webapp)
+        scene = Scene(instrument.telescope.tel_name, input=scene_configuration, webapp=webapp)
         warnings.update(scene.warnings)
 
     my_detector_signal_list, my_detector_noise_list, my_detector_saturation_list, obs = make_observation(calc_config, instrument, 
                                                                        strategy, scene, background, background_level, webapp=webapp)
+    if strategy.method=="taspec":
+        # The TASpec strategy uses one DetectorSignal and DetectorNoise, but 
+        # multiple extraction boxes. We need to make copies of the DetectorSignals.
+        my_detector_signal_list *= strategy.num_segments
+        my_detector_noise_list *= strategy.num_segments
+        my_detector_saturation_list *= strategy.num_segments
 
     # Use the strategy to get the extracted signal/noise products
     extracted_sn_list = strategy.extract(my_detector_signal_list, my_detector_noise_list)
@@ -343,40 +349,52 @@ def calculate_exposure_time(input, webapp=False, **kwargs):
     if contrast:
         scene, strategy = configure_coronagraphy(input, strategy, scene_configuration, warnings, webapp=webapp)
     else:
-        scene = Scene(input=scene_configuration, webapp=webapp)
+        scene = Scene(instrument.telescope.tel_name, input=scene_configuration,  webapp=webapp)
         warnings.update(scene.warnings)
 
     my_detector_signal_list, my_detector_noise_list, my_detector_saturation_list, obs = make_observation(calc_config, instrument, 
                                                                        strategy, scene, background, background_level, webapp=webapp)
 
+    if strategy.method=="taspec":
+        # The TASpec strategy uses one DetectorSignal and DetectorNoise, but 
+        # multiple extraction boxes. We need to make copies of the DetectorSignals.
+        my_detector_signal_list = [copy.deepcopy(my_detector_signal_list[0]) for x in range(strategy.num_segments)]
+        my_detector_noise_list = [copy.deepcopy(my_detector_noise_list[0]) for x in range(strategy.num_segments)]
+        my_detector_saturation_list = [copy.deepcopy(my_detector_saturation_list[0]) for x in range(strategy.num_segments)]
+
     # Use the strategy to get the extracted signal/noise products
     extracted_sn_list = strategy.extract(my_detector_signal_list, my_detector_noise_list)
     warnings.update(extracted_sn_list[0]['warnings'])
 
-    new_time = my_detector_signal_list[0].the_detector.scale_exposure_time(extracted_sn_list[0], input)
-    obs.instrument.the_detector.set_time(new_time)
-
-    for signal in range(len(my_detector_signal_list)):
-        # reset the time(s)
-        my_detector_signal_list[signal].the_detector.set_time(new_time)
-        my_detector_signal_list[signal].instrument.the_detector.set_time(new_time)
-        
-        # retrigger calculations using the new time
-        my_detector_signal_list[signal].trigger_saturation_calcs()
-        my_detector_noise_list[signal] = DetectorNoise(my_detector_signal_list[signal], obs)
-        my_detector_saturation_list[signal] = my_detector_signal_list[signal].get_saturation_mask()
-
-
-    # Use the strategy to get the extracted signal/noise products
-    extracted_sn_list = strategy.extract(my_detector_signal_list, my_detector_noise_list)
-
-    # selection logic: If calculation_config specifies it, do that. If not, use the predefined data value
+    # Detector Scattering.  If calculation_config specifies to apply it, do that. If not, use the
+    # predefined data value.
+    # 
+    # This detector effect is unusual for two reasons: One, it's defined as a fraction of the rate, but must be added
+    # to the rate_plus_bg; Two, it's only present in the noise.
     scatter = instrument.the_detector.scatter
     if calc_config.noise['scatter'] is not None:
         scatter = calc_config.noise['scatter']
     if scatter:
         extracted_sn_list = instrument.apply_scattering(extracted_sn_list)
 
+    for signal in range(len(my_detector_signal_list)):
+        new_time = my_detector_signal_list[signal].the_detector.scale_exposure_time(extracted_sn_list[signal], input)
+        obs.instrument.the_detector.set_time(new_time)
+        # reset the time(s)
+        my_detector_signal_list[signal].the_detector.set_time(new_time)
+        my_detector_signal_list[signal].instrument.the_detector.set_time(new_time)
+      
+        
+        # retrigger calculations using the new time
+        my_detector_signal_list[signal].signal_products()
+        my_detector_noise_list[signal] = DetectorNoise(my_detector_signal_list[signal], obs)
+        my_detector_saturation_list[signal] = my_detector_signal_list[signal].get_saturation_mask()
+
+    # Use the strategy to get the extracted signal/noise products
+    extracted_sn_list = strategy.extract(my_detector_signal_list, my_detector_noise_list)
+
+    if scatter:
+        extracted_sn_list = instrument.apply_scattering(extracted_sn_list)
 
     if contrast:
         extracted_sn_list, warnings = calculate_contrast(my_detector_signal_list, my_detector_noise_list, extracted_sn_list, strategy, warnings, webapp=webapp)
