@@ -206,8 +206,8 @@ class Extinction(DefaultConfig):
             # This simplification is good enough for now and matches what's most commonly used.  However, for complicated
             # SEDs subject to large amounts of complicated extinction, the more formal approach will be required.
             # The work required for this is described in Issue #1884.
-            bandpass_normalized_c_ext = sum(np.multiply(bp_throughput_interp, self.extinction_curve)) / \
-                                        sum(bp_throughput_interp)
+            bandpass_normalized_c_ext = np.sum(np.multiply(bp_throughput_interp, self.extinction_curve)) / \
+                                        np.sum(bp_throughput_interp)
 
             # Calculate A_lambda
             # see Issue #503#issuecomment-232136894
