@@ -3,6 +3,7 @@
 # basics
 import glob, json, os, pprint, sys
 import numpy as np
+from multiprocessing import Pool
 
 # pyetc tools  (hopefully not too many here)
 import pysynphot
@@ -16,7 +17,7 @@ from pandeia.engine.calc_utils import build_default_calc
 from pandeia.engine.perform_calculation import perform_calculation
 from pandeia.engine.io_utils import NumPyArangeEncoder
 from pandeia.engine.instrument_factory import InstrumentFactory
-from pandeia_test import utils # for rglob - maybe eventually move that to pandeia
+from pandeia_test import utils, data_util # for rglob - maybe eventually move that to pandeia
 
 
 # doc
@@ -42,7 +43,11 @@ HST_DETECTORS = ['ccd','ir','fuv','nuv','fuvmama','nuvmama','sbc','uvis1','uvis2
 HST_MODES = ['imaging','spectroscopic', 'spectroscopicacq', 'targetacquisition', 'scimaging', 'scspectroscopic', 'rampfilter']
 HST_MIRRORS = ['mirrora','mirrorb']
 
-MODE_NAME_DICT = {'spectroscopic': 'spectroscopy', 'scspectroscopic': 'scan_spectroscopy', 'scimaging': 'scan_imaging'}
+MODE_NAME_DICT = {'spectroscopic': 'spec',
+                  'spectroscopicacq': 'specacq',
+                  'scspectroscopic': 'scan_spec',
+                  'scimaging': 'scan_imaging',
+                  'targetacquisition': 'target_acq'}
 
 DETECTOR_INSTRUMENT_DICT = {'wfc': 'acs',
                             'hrc': 'acs',
@@ -56,6 +61,14 @@ DETECTOR_INSTRUMENT_DICT = {'wfc': 'acs',
                             'uvis1': 'wfc3',
                             'uvis2': 'wfc3'}
 
+APERTURE_DICT = {'psa':
+                     {'fuv': 'cosfuv',
+                      'nuv': 'cosnuv'},
+                 'wfc':
+                     {
+                        'wfc': 'wfc1'
+                     }
+                 }
 
 # Dictionaries to convert the listed flux/wavelength units in the peng file to their corresponding astropy/pysynphot units.
 FLUX_UNIT_DICT = {'photlam':syn.units.PHOTLAM,
@@ -230,13 +243,18 @@ def detector_parse(adict, calculation, pweb_key):
     else:
         calculation['configuration']['instrument']['mode'] = f"{adict['extraction']['detector']}_{mode_name}"
 
-
     calculation['configuration']['instrument']['aperture'] = adict['extraction']['detector']
     if 'aperture' in adict:
         if adict['aperture'] != adict['extraction']['detector'] and adict['instrument'] != 'stis':
             # STIS pengs have the filter in the aperture keyword.  Just leave aperture as extraction/detector.
             calculation['configuration']['instrument']['aperture'] = adict['aperture']
     assert adict['detector'] == adict['extraction']['detector']
+
+    # specific for COS spec acq mode
+    if 'cos' in adict['instrument'] and 'spectroscopicacq' in adict['science_mode']:
+        calculation['configuration']['detector']['gain'] = "1"
+        calculation['configuration']['detector']['nsplit'] = 1
+
     return calculation
 
 def obswave_parse(adict, calculation, pweb_key):
@@ -286,7 +304,16 @@ def obsmode_parse(adict, calculation, pweb_key):
             calculation['configuration']['instrument']['detector'] = obsmode_list[1].lower()
             calculation['configuration']['instrument']['disperser'] = obsmode_list[2].lower()
             calculation['configuration']['instrument']['cenwave'] = obsmode_list[3].lower()
-            calculation['configuration']['instrument']['aperture'] = obsmode_list[4].lower()
+
+            if obsmode_list[4].lower() in APERTURE_DICT:
+                calculation['configuration']['instrument']['aperture'] = \
+                    APERTURE_DICT[obsmode_list[4].lower()][adict['detector']]
+            else:
+                calculation['configuration']['instrument']['aperture'] = obsmode_list[4].lower()
+
+            if 'spectroscopicacq' in adict['science_mode']:
+                calculation['strategy']['method'] = 'taspec'
+
         elif ('targetacquisition' in adict['science_mode']) or ('imaging' in adict['science_mode']):
             """
             instrument,mirror,detector,aperture
@@ -411,23 +438,37 @@ def obsmode_parse(adict, calculation, pweb_key):
             polarizers. (of course, given that neither ETC intends to actually implement 
             polarization, they are effectively filters)
             """
+            det = adict['detector']
+            if det in APERTURE_DICT:
+                calculation['configuration']['instrument']['aperture'] = \
+                    APERTURE_DICT[det][det]
+                calculation['configuration']['instrument']['detector'] = \
+                    APERTURE_DICT[det][det]
+            else:
+                calculation['configuration']['instrument']['aperture'] = obsmode_list[1].lower()
             calculation['configuration']['instrument']['detector'] = obsmode_list[1].lower()
-            if obsmode_list[1] == 'wfc1':
-                calculation['configuration']['instrument']['detector'] = 'wfc'
 
             # According to ACS Handbook, dispersers start with 'G8' or 'PR'.
             # Polarizers start with 'POL'
             # Everything else will be assumed to be a filter.
 
             # Loop through the remaining obsmode_list items, and determine if they are filter/polarizer/disperser...
+            filter_first = ''
+            filter_second = ''
             for i in range(len(obsmode_list) - 2): #[0] and [1] are instrument and detector, so skip the first 2 items.
                 index = i + 2
                 if obsmode_list[index][:2] in ['g8', 'pr']:
-                    calculation['configuration']['instrument']['disperser'] = obsmode_list[index].lower()
+                    filter_first = obsmode_list[index].lower()
                 elif str.startswith(obsmode_list[index], 'pol'):
-                    calculation['configuration']['instrument']['polarizer'] = obsmode_list[index].lower()
+                    filter_second = obsmode_list[index].lower()
                 else:
-                    calculation['configuration']['instrument']['filter'] = obsmode_list[index].lower()
+                    filter_first = obsmode_list[index].lower()
+
+            if filter_second != '':
+                calculation['configuration']['instrument']['disperser'] = filter_first + ',' + filter_second
+            else:
+                calculation['configuration']['instrument']['disperser'] = filter_first
+
         elif 'ramp' in adict['science_mode']:
             """
             instrument,detector,filter#wave
@@ -463,27 +504,25 @@ def obsmode_parse(adict, calculation, pweb_key):
             have a case where both are clear and the obsmode string ends at “detector”. Using 
             clear in both wheels is allowed, but generates a not-supported warning.
             """
-            calculation['configuration']['instrument']['detector'] = obsmode_list[1].lower()
-            if calculation['configuration']['instrument']['detector'] == 'wfc1':
-                calculation['configuration']['instrument']['detector'] = 'wfc'
-            if len(obsmode_list) == 2:
-                calculation['configuration']['instrument']['filter'] = adict['filter'].lower()
-            if len(obsmode_list) == 3:
-                if "pol" in obsmode_list[2]:
-                    calculation['configuration']['instrument']['polarizer'] = obsmode_list[2].lower()
-                else:
-                    calculation['configuration']['instrument']['filter'] = obsmode_list[2].lower()
-            if len(obsmode_list) == 4:
-                if "pol" in obsmode_list[3]:
-                    calculation['configuration']['instrument']['filter'] = obsmode_list[2].lower()
-                    calculation['configuration']['instrument']['polarizer'] = obsmode_list[3].lower()
-                else:
-                    calculation['configuration']['instrument']['filter'] = obsmode_list[3].lower()
-                    calculation['configuration']['instrument']['polarizer'] = obsmode_list[2].lower()
+            detector = obsmode_list[1].lower()
+            if detector == 'wfc':
+                detector = 'wfc1'
+            calculation['configuration']['instrument']['detector'] = detector
+            calculation['configuration']['instrument']['aperture'] = detector
+
+            calculation['configuration']['instrument']['filter'] = obsmode_list[2]
+            if len(obsmode_list) > 3:
+                calculation['configuration']['instrument']['filter'] += ',' + obsmode_list[3]
 
     return calculation
 
 
+ACQ_MODE_TRANSLATION = {
+    'ACQ': 'acq_search',
+    'ACQ/PEAKD': 'acq_peakd',
+    'ACQ/PEAKXD': 'acq_peakxd'
+}
+
 def extraction_parse(adict, calculation, pweb_key):
     calculation['strategy']['gyromode'] = adict['extraction']['gyromode']
     calculation['configuration']['instrument']['detector'] = adict['extraction']['detector']
@@ -510,16 +549,17 @@ def extraction_parse(adict, calculation, pweb_key):
 
     if 'uvis' in adict['extraction']['detector']:
         inst_config['instrument']['mode'] = f"uvis_{mode_name}"
+    elif 'ir' in adict['extraction']['detector']:
+        inst_config['instrument']['mode'] = f"ir_{mode_name}"
     elif instrument == 'stis' and 'spec' in adict['science_mode']:
         if adict['grating'].startswith('e'):
             # Echelle mode
             inst_config['instrument']['mode'] = f"{detector}_echelle"
-        elif adict['grating'].startswith('g'):
-            # 1st order spec mode.  Now determine if slit or slitless.
-            slit = "slit"
-            if adict['aperture'].startswith('f') or adict['aperture'] == "50ccd" or adict['aperture'] == "25mama":
-                slit = "slitless"
-            inst_config['instrument']['mode'] = f"{detector}_{mode_name}_{slit}"
+        elif adict['grating'].startswith('g') or adict['grating'] == "prism":
+            # 1st order spec mode.  Slit and slitless are in the same modes
+            inst_config['instrument']['mode'] = f"{detector}_{mode_name}"
+    elif instrument == 'cos' and 'spec' in adict['science_mode'] and detector == 'fuv':
+        inst_config['instrument']['mode'] = f"{detector}_spec"
     else:
         inst_config['instrument']['mode'] = f"{detector}_{mode_name}"
 
@@ -543,12 +583,33 @@ def extraction_parse(adict, calculation, pweb_key):
 
     # Use the shape parameter and the science_mode parameter to determine the resulting strategy in the JENG file.
     if adict['extraction']['shape'] == 'rectangle' and 'spectro' not in adict['science_mode']:
-        calculation['strategy']['method'] = 'imagingrectphot'
+        if 'scimag' in adict['science_mode']:
+            calculation['strategy']['method'] = 'imagingscanapphot'
+            if adict['extraction']['size'][1] == 1.0:
+                calculation['strategy']['scan_extraction_length'] = 'pixel'
+            else:
+                calculation['strategy']['scan_extraction_width'] = 'scanlength'
+            calculation['strategy']['scan_extraction_width'] = adict['extraction']['size'][0]
+        else:
+            calculation['strategy']['method'] = 'imagingrectphot'
+            calculation['strategy']['display_string'] = "Imaging Rectangular Photometry"
         calculation['strategy']['aperture_size'] = [adict['extraction']['size'][0] * plate_scale_x, adict['extraction']['size'][1] * plate_scale_y]
-        calculation['strategy']['display_string'] = "Imaging Rectangular Photometry"
     elif adict['extraction']['shape'] == 'rectangle':
-        calculation['strategy']['method'] = 'specapphot' # note that SpecApPhot only accepts the half-height of the box, and relies on other means to get the length of the extraction box
-        calculation['strategy']['aperture_size'] = (adict['extraction']['size'][0] * plate_scale_x, adict['extraction']['size'][1] * plate_scale_y)
+        if 'scspectro' in adict['science_mode']:
+            calculation['strategy']['method'] = 'specscanapphot'
+            if adict['extraction']['size'][0] == 1.0 and adict['extraction']['size'][1] == 1.0:
+                calculation['strategy']['scan_extraction_length'] = "pixel"
+            else:
+                calculation['strategy']['scan_extraction_length'] = "scanlength"
+        else:
+            if adict['instrument'] == 'cos' and adict['detector'] == 'fuv':
+                calculation['strategy']['method'] = 'specsegmentedphot'
+            else:
+                calculation['strategy']['method'] = 'specapphot' # note that SpecApPhot only accepts the half-height of the box, and relies on other means to get the length of the extraction box
+        if 'spectro' in adict['science_mode']:
+            calculation['strategy']['aperture_size'] = adict['extraction']['size'][0] * plate_scale_x
+        else:
+            calculation['strategy']['aperture_size'] = (adict['extraction']['size'][0] * plate_scale_x, adict['extraction']['size'][1] * plate_scale_y)
         calculation['strategy']['display_string'] = "Aperture Spectral Extraction"
     if adict['extraction']['shape'] == 'square':
         calculation['strategy']['method'] = 'imagingrectphot'
@@ -563,7 +624,7 @@ def extraction_parse(adict, calculation, pweb_key):
         # TODO: by default, the units are 'ee' (encircled energy) which is currently not implemented in pandeia.  Once those tickets are completed, we can get rid of the conversions here and just use 'ee'.
         calculation['strategy']['units'] = 'arcsec'
     if 'stripe' in adict['extraction']:
-        calculation['strategy']['order'] = adict['extraction']['stripe']
+        calculation['strategy']['stripe'] = adict['extraction']['stripe'].lower()
     if adict['extraction']['stype'] == 'extended':
         # According to Ivo, sdiameter and fdiameter are the same, and pyetc uses compatibility code like this
         try:
@@ -576,6 +637,11 @@ def extraction_parse(adict, calculation, pweb_key):
         calculation['scene'][0]['shape'] = {"geometry": "flat","major": diameter, "minor": diameter,
             "norm_method": "integ_infinity", "surf_area_units": None }
 
+    # COS spec acq mode uses an extra parameter, acq_mode
+    if 'acq_mode' in adict['extraction']:
+        acq_mode = adict['extraction']['acq_mode']
+        calculation['strategy']['acq_mode'] = ACQ_MODE_TRANSLATION[acq_mode]
+
     return calculation
 
 def simmode_valid(adict, calculation):
@@ -921,7 +987,12 @@ def background_parse(adict, calculation, pweb_key):
 
     return calculation
 
-parsefuncs = [dummy_parse, detector_parse, extraction_parse, obsmode_parse, obswave_parse, sourceval_parse, lineval_parse, background_parse]
+def gain_parse(adict, calculation, pweb_key):
+    calculation['configuration']['detector']['gain'] = str(adict['gain'])
+
+    return calculation
+
+parsefuncs = [dummy_parse, detector_parse, extraction_parse, obsmode_parse, obswave_parse, sourceval_parse, lineval_parse, background_parse, gain_parse]
 validfuncs = [disperser_valid, simmode_valid]
 
 H2J_INPUT_CHECKS = {
@@ -943,7 +1014,7 @@ H2J_INPUT_CHECKS = {
     'fdiameter':           [dummy_parse,                                        'TYPE:NUM',        False ],
     'filter':              [('configuration', 'instrument','filter'),           'TYPE:STR',        False ],
     'fuvglowregion':       [('configuration','detector','fuv_glow_region'),     'TYPE:STR',        True ],
-    'gain':                [('configuration','detector','gain'),                'TYPE:NUM',        False ],
+    'gain':                [gain_parse,                                         'TYPE:NUM',        False ],
     'grating':             [('configuration','instrument','disperser'),         'TYPE:STR',        False ],
     'helium_str':          [dummy_parse,                                        'TYPE:STR',        True ],
     'instrument':          [('configuration','instrument','instrument'),        HST_INSTS,         False ],
@@ -951,10 +1022,10 @@ H2J_INPUT_CHECKS = {
     'obsmode':             [obsmode_parse,                                      'TYPE:STR',        False ],
     'mirror':              [dummy_parse,                                        HST_MIRRORS,       False ],
     'obswave':             [obswave_parse,                                      'TYPE:NUM',        False],
-    'post_flash_electrons':[('configuration','detector','post_flash_electrons'),'TYPE:INT',        False ],
+    'post_flash_electrons':[('configuration','detector','postflash'),           'TYPE:INT',        False ],
     'science_mode':        [('configuration','instrument','mode'),              HST_MODES,         False ],
-    'sclength':            [('configuration','instrument','scan_length'),       'TYPE:NUM',        False ],
-    'scrate':              [('configuration','instrument','scan_rate'),         'TYPE:NUM',        False ],
+    'sclength':            [('configuration','detector','scan_length'),         'TYPE:NUM',        False ],
+    'scrate':              [('configuration','detector','scan_rate'),           'TYPE:NUM',        False ],
     'simmode':             [simmode_valid,                                      'TYPE:STR',        True ], # might force these for a while to pick up more stis tests
     'sky_expr':            [background_parse,                                   'TYPE:STR',        True ],
     'source_expr':         [sourceval_parse,                                    'TYPE:STR',        False ],
@@ -1007,6 +1078,7 @@ def pyetc_in_dict_to_pandeia_in_dict(adict, input_fname, pweb_dict=None):
 
     # instrument and mode, filters and dispersers
     calculation = build_default_calc("jwst", "nircam", "sw_imaging")
+    calculation = data_util.clean(calculation)
     calculation['background'] = {}
     # some calculations have no filter or disperser
     calculation['configuration']['instrument']['filter'] = None
@@ -1130,31 +1202,26 @@ def run_some_samples(pyetc_test_dir, top_out_dir, single_file=None):
             flist += sublist
 
     done = {}
-    for fname in flist:
-        # outstr = convert_file(fname, top_out_dir)#, limit_outputs=done)
-        # # if no exception than print the ouput
-        # print(outstr)
-        # instdotmode = get_instdotmode_from_output(outstr)
-        # if instdotmode in done:
-        #     done[instdotmode] += 1
-        # else:
-        #     done[instdotmode] = 1
-        try:
-            outstr = convert_file(fname, top_out_dir)#, limit_outputs=done)
-            # if no exception than print the ouput
-            instdotmode = get_instdotmode_from_output(outstr)
-            if instdotmode in done:
-                done[instdotmode] += 1
-            else:
-                done[instdotmode] = 1
-        except Exception as e:
-            if len(flist) == 1:
-                raise e
-            reason = str(e) if type(e) == AssertionError else repr(e)
-            if reason in skips:
-                skips[reason].append(fname)
-            else:
-                skips[reason] = [fname]
+    with Pool(7) as p:
+        for output in p.imap_unordered(convert_file_helper, zip(flist, [top_out_dir]*len(flist))):
+            if output[0] == "Success":
+                instdotmode = output[1]
+                fname = output[2]
+                if instdotmode in done:
+                    done[instdotmode] += 1
+                else:
+                    done[instdotmode] = 1
+                print(f"{fname} Success")
+            elif output[0] == "Failure":
+                if len(flist) == 1:
+                    raise output[3]
+                reason = output[1]
+                fname = output[2]
+                if reason in skips:
+                    skips[reason].append(fname)
+                else:
+                    skips[reason] = [fname]
+                print(f"{fname} Failure ({reason}")
 
     total_skipped = 0
     if skips:
@@ -1166,6 +1233,19 @@ def run_some_samples(pyetc_test_dir, top_out_dir, single_file=None):
 
     print('\nConverted %d and skipped %d tests. Of the converted: %s' % (len(flist)-total_skipped, total_skipped, done))
 
+def convert_file_helper(convertinput):
+    fname, top_out_dir = convertinput
+    try:
+        outstr = convert_file(fname, top_out_dir)#, limit_outputs=done)
+        instdotmode = get_instdotmode_from_output(outstr)
+        status = "Success"
+        output = (status, instdotmode, fname)
+    except Exception as e:
+        reason = str(e) if type(e) == AssertionError else repr(e)
+        status = "Failure"
+        output = (status, reason, fname, e)
+    return output
+
 
 #
 # main routine
@@ -1174,7 +1254,7 @@ if __name__=='__main__': # in case something else imports this file
     # If this is run with no args, go fetch a same set of tests
     if len(sys.argv) == 1:
         run_some_samples(os.path.expanduser(f'{DATA_DIR}/../pyetc/test'),
-                         os.path.expanduser(f'{DATA_DIR}/../pandeia_test/tests/engine/hst'))
+                         os.path.expanduser(f'{DATA_DIR}/../pandeia_test/tests/engine_converted/hst'))
 
     # If this is given a peng file name as an arg on the command line, translate it
     else:
