@@ -33,6 +33,7 @@ EMPTY_SCHEMA = {
 
 # This regex matches a string representation of an integer.
 INT_REGEX = '^-?[0-9]+$'
+FLOAT_REGEX = '^-?[0-9]+\.?[0-9]*$'
 
 #-------------------------------------------------------------------------------
 # DEFINITIONS OF PARTS - these are not to be used directly during validation,
@@ -202,7 +203,7 @@ a_background_hst = {
     'additionalProperties': False,
     'properties': {
         'airglow': {
-            'type': 'object', 
+            'type': 'object',
             'required': ['choice', 'standard'],
             'additionalProperties': False,
             'properties': {
@@ -211,26 +212,36 @@ a_background_hst = {
             }
         },
         'earthshine': {
-            'type': 'object', 
-            'required': ['choice', 'standard'],
+            'type': 'object',
+            'required': ['choice'],
             'additionalProperties': False,
             'properties': {
-                'choice':     {'type': 'string', 'enum': ['standard']},
-                'standard':   {'type': 'string', 'enum': ['none', 'average', 'high', 'extremely_high']}
+                'choice':     {'type': 'string', 'enum': ['standard', 'magnitude', 'multiplier']},
+                'standard':   {'type': 'string', 'enum': ['none', 'average', 'high', 'extremely_high']},
+                'magnitude':  {'type': 'number'},
+                'multiplier': {'type': 'number'}
             }
         },
         'zodiacallight':  {
-            'type': 'object', 
-            'required': ['choice', 'standard'],
+            'type': 'object',
+            'required': ['choice'],
             'additionalProperties': False,
             'properties': {
-                'choice':     {'type': 'string', 'enum': ['standard']},
-                'standard':   {'type': 'string', 'enum': ['none', 'low', 'average', 'high']}
+                'choice':     {'type': 'string', 'enum': ['standard', 'magnitude', 'multiplier']},
+                'standard':   {'type': 'string', 'enum': ['none', 'low', 'average', 'high']},
+                'magnitude':  {'type': 'number'},
+                'multiplier': {'type': 'number'}
             }
         }
     }
 }
 
+a_background_hst_ir = copy.deepcopy(a_background_hst)
+a_background_hst_ir['properties']['helium'] = a_background_hst_ir['properties']['airglow']
+del a_background_hst_ir['properties']['airglow']
+a_background_hst_ir['properties']['helium']['properties']['standard']['enum'] = ['none', 'average', 'high', 'very_high']
+a_background_hst_ir['required'] = ['earthshine','helium','zodiacallight']
+
 a_background_none = {
     'type': 'object',
     'required': ['bg_type'],
@@ -268,7 +279,7 @@ a_background_dated = {
 
 a_background_choice = {
     'type': 'object',
-    'oneOf': [a_background_hst, a_background_none, a_background_dateless, a_background_dated],
+    'oneOf': [a_background_hst, a_background_hst_ir, a_background_none, a_background_dateless, a_background_dated],
 }
 
 a_dither_simple = {
@@ -350,7 +361,7 @@ a_strategy_coronagraphy = {
                 'target_source':          {'type': 'string', 'pattern': INT_REGEX},
                 'target_type':            {'type': 'string', 'enum': ['source']},
                 'strat_offset_xy':        {'type': 'array', 'items': {'type': 'number'}},
-                'units':                  {'type': 'string', 'enum': ['arcsec']},
+                'units':                  {'type': 'string', 'enum': ['arcsec', 'pixel']},
             },
         },
         a_helper_strategy_background,
@@ -369,7 +380,7 @@ a_strategy_ifunodinscene = {
         'target_source':        {'type': 'string', 'pattern': INT_REGEX},
         'target_type':          {'type': 'string', 'enum': ['coords','source']},
         'strat_offset_xy':      {'type': 'array', 'items': {'type': 'number'}},
-        'units':                {'type': 'string', 'enum': ['arcsec']},
+        'units':                {'type': 'string', 'enum': ['arcsec', 'pixel']},
     },
 }
 
@@ -385,7 +396,7 @@ a_strategy_ifunodoffscene = {
         'target_source':        {'type': 'string', 'pattern': INT_REGEX},
         'target_type':          {'type': 'string', 'enum': ['coords','source']},
         'strat_offset_xy':      {'type': 'array', 'items': {'type': 'number'}},
-        'units':                {'type': 'string', 'enum': ['arcsec']},
+        'units':                {'type': 'string', 'enum': ['arcsec', 'pixel']},
     },
 }
 
@@ -400,7 +411,26 @@ a_strategy_imagingapphot = {
                 'strat_offset_xy': {'type': 'array', 'items': {'type': 'number'}},
                 'target_source':   {'type': 'string', 'pattern': INT_REGEX},
                 'target_type':     {'type': 'string', 'enum': ['coords','source']},
-                'units':           {'type': 'string', 'enum': ['arcsec']},
+                'units':           {'type': 'string', 'enum': ['arcsec','pixel']},
+            },
+        },
+        a_helper_strategy_background,
+    ],
+}
+
+a_strategy_imagingscanapphot = {
+    'allOf': [
+        {
+            'type': 'object',
+            'required': ['method','scan_extraction_length','scan_extraction_width','strat_offset_xy','target_source','target_type','units'],
+            'properties': {
+                'method':                 {'type': 'string', 'enum': ['imagingscanapphot']},
+                'scan_extraction_length': {'type': 'string', 'enum': ['pixel','scanlength']},
+                'scan_extraction_width':  {'type': 'number'},
+                'strat_offset_xy':        {'type': 'array', 'items': {'type': 'number'}},
+                'target_source':          {'type': 'string', 'pattern': INT_REGEX},
+                'target_type':            {'type': 'string', 'enum': ['coords','source']},
+                'units':                  {'type': 'string', 'enum': ['arcsec','pixel']},
             },
         },
         a_helper_strategy_background,
@@ -419,7 +449,7 @@ a_strategy_msaapphot = {
                 'strat_offset_xy': {'type': 'array', 'items': {'type': 'number'}},
                 'target_source':   {'type': 'string', 'pattern': INT_REGEX},
                 'target_type':     {'type': 'string', 'enum': ['source']},
-                'units':           {'type': 'string', 'enum': ['arcsec']},
+                'units':           {'type': 'string', 'enum': ['arcsec', 'pixel']},
             },
         },
         a_helper_strategy_background,
@@ -438,7 +468,7 @@ a_strategy_msafullapphot = {
         'target_source':        {'type': 'string', 'pattern': INT_REGEX},
         'target_type':          {'type': 'string', 'enum': ['source']},
         'strat_offset_xy':      {'type': 'array', 'items': {'type': 'number'}},
-        'units':                {'type': 'string', 'enum': ['arcsec']},
+        'units':                {'type': 'string', 'enum': ['arcsec', 'pixel']},
     },
 }
 
@@ -472,7 +502,40 @@ a_strategy_specapphot = {
                 'target_source':        {'type': 'string', 'pattern': INT_REGEX},
                 'target_type':          {'type': 'string', 'enum': ['coords','source']},
                 'strat_offset_xy':      {'type': 'array', 'items': {'type': 'number'}},
-                'units':                {'type': 'string', 'enum': ['arcsec']},
+                'units':                {'type': 'string', 'enum': ['arcsec','pixel']},
+            },
+        },
+        a_helper_strategy_background,
+    ],
+}
+
+# specsegmentedphot is just specapphot without aperture_size
+a_strategy_specsegmentedphot = copy.deepcopy(a_strategy_specapphot)
+a_strategy_specsegmentedphot['allOf'][0]['required'].remove('aperture_size')
+del a_strategy_specsegmentedphot['allOf'][0]['properties']['aperture_size']
+a_strategy_specsegmentedphot['allOf'][0]['properties']['method']['enum'] = ['specsegmentedphot']
+
+# taspec is just specsegmentedphot without reference_wavelength but with acq_mode and stripe (those optional since fuv does not use)
+a_strategy_taspec = copy.deepcopy(a_strategy_specsegmentedphot)
+a_strategy_taspec['allOf'][0]['required'].remove('reference_wavelength')
+del a_strategy_taspec['allOf'][0]['properties']['reference_wavelength']
+a_strategy_taspec['allOf'][0]['properties']['method']['enum'] = ['taspec']
+a_strategy_taspec['allOf'][0]['properties']['acq_mode'] = {'type': 'string', 'enum': ['acq_search', 'acq_peakd', 'acq_peakxd']}
+a_strategy_taspec['allOf'][0]['properties']['stripe'] = {'type': 'string', 'enum': ['a', 'b', 'c']}
+
+a_strategy_specscanapphot = {
+    'allOf': [
+        {
+            'type': 'object',
+            'required': ['method','reference_wavelength','scan_extraction_length','target_source','target_type','strat_offset_xy','units'],
+            'properties': {
+                'method':                 {'type': 'string', 'enum': ['specscanapphot']},
+                'reference_wavelength':   {'oneOf': [{'type': 'number'}, {'type': 'null'}]},
+                'scan_extraction_length': {'type': 'string', 'enum': ['pixel','scanlength']},
+                'target_source':          {'type': 'string', 'pattern': INT_REGEX},
+                'target_type':            {'type': 'string', 'enum': ['coords','source']},
+                'strat_offset_xy':        {'type': 'array', 'items': {'type': 'number'}},
+                'units':                  {'type': 'string', 'enum': ['arcsec','pixel']},
             },
         },
         a_helper_strategy_background,
@@ -494,8 +557,10 @@ a_strategy_taphot = {
 
 a_strategy_choice = {
     'type': 'object',
-    'oneOf': [a_strategy_coronagraphy, a_strategy_ifunodinscene, a_strategy_ifunodoffscene, a_strategy_imagingapphot, a_strategy_msaapphot,
-              a_strategy_msafullapphot, a_strategy_msashutterapphot, a_strategy_soss, a_strategy_specapphot, a_strategy_taphot],
+    'oneOf': [a_strategy_coronagraphy, a_strategy_ifunodinscene, a_strategy_ifunodoffscene, a_strategy_imagingapphot,
+              a_strategy_imagingscanapphot, a_strategy_msaapphot, a_strategy_msafullapphot, a_strategy_msashutterapphot,
+              a_strategy_soss, a_strategy_specapphot, a_strategy_specscanapphot, a_strategy_specsegmentedphot,
+              a_strategy_taspec, a_strategy_taphot],
 }
 
 a_scene_element = {
@@ -599,10 +664,13 @@ a_detector_hst = {
         'bin_spatial':     {'type': 'number'}, # optional
         'calculate_snr':   {'type': 'boolean'},
         'dark_level':      {'type': 'string'}, # optional
-        'gain':            {'type': 'number'}, # optional
+        'gain':            {'type': 'string', 'pattern': FLOAT_REGEX}, # optional
         'fuv_glow_region': {'type': 'string'}, # optional
         'nexp':            {'type': 'number'}, # optional
         'nsplit':          {'type': 'number'}, # optional
+        'postflash':       {'type': 'number'}, # optional
+        'scan_choice':     {'type': 'string', 'enum': ['rate', 'time']}, # optional
+        'scan_length':     {'type': 'number'}, # optional
         'snr':             {'type': 'number'},
         'time':            {'type': 'number'},
     },
@@ -640,9 +708,10 @@ a_camera_config = {
                 'aperture':         {'type': 'string'},
                 'cenwave':          {'type': 'string'}, # optional, HST only, may break all this up
                 'slit':             {'type': 'string'}, # optional, HST only, may break all this up
-                'detector':         {'type': 'string'}, # optional
+                'detector':         {'type': 'string'}, # optional, both HST and JWST use
                 'disperser':        {'oneOf': [{'type': 'string'}, {'type': 'null'}]},
                 'filter':           {'oneOf': [{'type': 'string'}, {'type': 'null'}]},
+                'paired_filter':    {'oneOf': [{'type': 'string'}, {'type': 'null'}]}, # optional, JWST only
                 'shutter_location': {'type': 'string'}, # optional
                 'slitlet_shape': {'type': 'string'} # optional
             }
@@ -732,6 +801,7 @@ WORKBOOK_SCHEMA = {
         'an_sed_powerlaw':              an_sed_powerlaw,
         'an_sed_keyed':                 an_sed_keyed,
         'a_background_hst':             a_background_hst,
+        'a_background_hst_ir':          a_background_hst_ir,
         'a_background_none':            a_background_none,
         'a_background_dateless':        a_background_dateless,
         'a_background_dated':           a_background_dated,
@@ -743,12 +813,16 @@ WORKBOOK_SCHEMA = {
         'a_strategy_ifunodinscene':     a_strategy_ifunodinscene,
         'a_strategy_ifunodoffscene':    a_strategy_ifunodoffscene,
         'a_strategy_imagingapphot':     a_strategy_imagingapphot,
+        'a_strategy_imagingscanapphot': a_strategy_imagingscanapphot,
         'a_strategy_msaapphot':         a_strategy_msaapphot,
         'a_strategy_msafullapphot':     a_strategy_msafullapphot,
         'a_strategy_msashutterapphot':  a_strategy_msashutterapphot,
         'a_strategy_soss':              a_strategy_soss,
         'a_strategy_specapphot':        a_strategy_specapphot,
+        'a_strategy_specscanapphot':    a_strategy_specscanapphot,
+        'a_strategy_specsegmentedphot': a_strategy_specsegmentedphot,
         'a_strategy_taphot':            a_strategy_taphot,
+        'a_strategy_taspec':            a_strategy_taspec,
         'a_strategy_choice':            a_strategy_choice,
 
         # scenes/sources
