@@ -30,6 +30,27 @@ def notin(i,l):
 
 def check_warnings(input_dict, warnings, warnings_list, value_lookup):
     """
+    Input/Output warnings checker.
+
+    The basic way this works is: the instrument config.json file has a warnings section. In it are a series of
+    parameters that are interpreted as conditional statements, covering both which instrument setup this
+    applies to AND the actual level/value we are checking for. We start with need_warning = True and evaluate
+    each condition statements. If any one of them is False, we set need_warning = False and no warning is
+    emitted. Warnings are only evaluated if every single one of the statements is True. Thus, we can filter by
+    mode, disperser, or filter AND a numerical value.
+
+    Values tested by the conditional are defined by the value_lookup dict as "shortname": [nested dict
+    location] and can be inserted into warning strings as $shortname. The values are only evaluated when the
+    warning conditions are parsed, so the dict can have parameters that are not defined for all instrument
+    setups without fear of crashing. 
+
+    There are two reserved words that must not show up as keys:
+    * $threshold: The threshold value that triggers the warning.
+    * $subthreshold: The fraction of the threshold value that triggers the subthreshold warning (converted to
+      a percentage for output)
+    Using these in warning messages should help enforce consistency: we don't have to update the message
+    every time the number is changed, nor do we have to worry about copy&paste errors.
+
     :param warnings_list: list of warnings to check for and add to the warnings list
     :param warnings: warnings dict that is updated by the function with new warnings
     :param input_dict: input as passed into the function
@@ -43,6 +64,7 @@ def check_warnings(input_dict, warnings, warnings_list, value_lookup):
         # Run through all the parameters for this warning to see if all are true
         need_warning = True
         thewarning = warning['warning_text']
+        threshold = None
         for key, expected_value in list(warning['parameters'].items()):
 
             # Get the value from the json dict. This does a lookup
@@ -74,6 +96,10 @@ def check_warnings(input_dict, warnings, warnings_list, value_lookup):
             else:
                 raise InternalError(value="Unknown operator on key {}__{}".format(key, operator_text))
 
+            if operator_text != "==":
+                # if this value was subject to a quantitative comparison, save it as the threshold value
+                threshold = expected_value
+
             # For some reason there started to be collisions on comparison of str vs unicode so I am
             # going to convert things to string that might be string or unicode.  That way the comparison
             # will work properly.  In Python3 this will likely be able to be removed.
@@ -86,6 +112,12 @@ def check_warnings(input_dict, warnings, warnings_list, value_lookup):
 
             # Compare the input_value with the expected value
             if (  # do operator comparison for value
+                ### TODO
+                ### this line of code prevents crashing when input_value is None
+                ### This was put in place to prevent errors with scan_rate.
+                ### This should be removed when scan_rate gets implemented.
+                ###
+                (input_value is not None) and
                 (isinstance(expected_value, (six.string_types, six.integer_types, float)) and cmp(input_value, expected_value)) or
                 # do set inclusion or exclusion for list
                     (isinstance(expected_value, list) and
@@ -94,17 +126,35 @@ def check_warnings(input_dict, warnings, warnings_list, value_lookup):
                     )
                 ):
                 pass  # we do need a warning, based on this test
+            elif "subthreshold" in warning and (isinstance(expected_value, (six.integer_types, float)) and
+                                                input_value is not None and
+                                                cmp(input_value, expected_value * warning['subthreshold'])):
+                ### And now, subthresholds. They are always percentages of a float/int.
+                # It makes no sense to have an == or notin operator for a subthreshold.
+
+                # If true, we do need the warning, but the subthreshold warning.
+                # Such warnings have the subthreshold percentage in the message; it's in a 
+                # Python list formatting syntax for inclusion here.
+                thewarning = warning['subthreshold_text']
             else:
                 need_warning = False
 
             # Replace $mode, $instrument, $filter etc in the output warning with the actual value.
             # special syntax if we don't want it to be uppercase.
-            input_value = str(input_value)
+            if isinstance(input_value, float):
+                input_value = f"{input_value:.1f}"
+            else:
+                input_value = str(input_value)
             if '$~'+key in thewarning:
                 thewarning = thewarning.replace('$~' + key, input_value)
             if '$'+key in thewarning:
                 thewarning = thewarning.replace('$' + key, input_value.upper())
-
+            if threshold is not None:
+                thewarning = thewarning.replace("$threshold", str(threshold))
+            # this will do nothing if there's no subthreshold keyword in the message. All values are
+            # fractions, and are expected to be percentages in the output
+            if "subthreshold" in warning:
+                thewarning = thewarning.replace("$subthreshold", f"{warning['subthreshold']*100:.0f}")
         if need_warning:
             warnings[thewarning] = thewarning
 
@@ -131,9 +181,9 @@ class InstrumentConfig(TelescopeConfig):
         config = read_json(os.path.join(self.ref_dir, "config.json"), raise_except=True)
 
         # Grab the warning lists, if they exists, in the config file.
-        self.warning_list = []
-        if 'warning_list' in config:
-            self.warning_list = config['warning_list']
+        self.warning_list_input = []
+        if 'warning_list_input' in config:
+            self.warning_list_input = config['warning_list_input']
         if 'warning_list_output' in config:
             self.warning_list_output = config['warning_list_output']
 
@@ -208,7 +258,6 @@ class Instrument(InstrumentConfig):
         self.read_detector()
 
         self.the_detector = self.get_detector_pars()
-        self._loadpsfs()
 
         # Check the instrument and strategy for the calculation against
         # the configured warnings in the instrument's config.json file.
@@ -227,15 +276,17 @@ class Instrument(InstrumentConfig):
                 'filter': ['instrument', 'filter'],
                 'disperser': ['instrument', 'disperser'],
                 'aperture': ['instrument', 'aperture'],
+                'detector': ['instrument', 'detector'],
                 'nexp': ['detector', 'nexp'],
                 'ngroup': ['detector', 'ngroup'],
                 'nint': ['detector', 'nint'],
+                'postflash': ['detector', 'postflash'],
                 'max_total_groups': ['detector', 'max_total_groups'],
                 'readout_pattern': ['detector', 'readout_pattern'],
                 'subarray': ['detector', 'subarray'],
             }
 
-            check_warnings(all_config, self.warnings, self.warning_list, value_lookup)
+            check_warnings(all_config, self.warnings, self.warning_list_input, value_lookup)
 
     def _nested_api_checks(self, conf, webapp=False):
         for section in self.instrument_pars:
@@ -279,7 +330,7 @@ class Instrument(InstrumentConfig):
     def _load_psf_library(self,psf_key):
 
         psf_path = os.path.join(self.ref_dir, "psfs")
-        library = PSFLibrary(path=psf_path, psf_key=psf_key)
+        library = PSFLibrary(self.get_wave_range(), path=psf_path, psf_key=psf_key)
 
         return library
 
@@ -366,7 +417,13 @@ class Instrument(InstrumentConfig):
         
         Binning will be an integer or 1 if not present
         """
-        return self.detector.get("bin_spatial", 1)
+
+        if (("bin_spatial" in self.detector) and not self.the_detector.detector_config.get("has_binning", False)):
+            raise EngineInputError(f"Detector does not support pixel binning.")
+        bin_spatial = self.detector.get("bin_spatial", 1)
+        if (not isinstance(bin_spatial, int)) or (bin_spatial < 1):
+            raise EngineInputError(f"Spatial binning value {bin_spatial} invalid. Must be positive integer.")
+        return bin_spatial
 
     def get_dispersion_binning(self):
         """
@@ -374,13 +431,19 @@ class Instrument(InstrumentConfig):
         
         Binning will be an integer or 1 if not present
         """
-        return self.detector.get("bin_dispersion", 1)
+
+        if (("bin_dispersion" in self.detector) and not self.the_detector.detector_config.get("has_binning", False)):
+            raise EngineInputError(f"Detector does not support pixel binning.")
+        bin_dispersion = self.detector.get("bin_dispersion", 1)
+        if (not isinstance(bin_dispersion, int)) or (bin_dispersion < 1):
+            raise EngineInputError(f"Dispersion binning value {bin_dispersion} invalid. Must be positive integer.")
+        return bin_dispersion
 
     def get_binning(self):
         """
         Combined binning for the data
         """
-        if self.projection_type=="imaging":
+        if self.projection_type in ["image", "image_scan"]:
             return self.detector.get("bin_spatial", 1) ** 2
         else:
             return self.detector.get("bin_spatial", 1) * self.detector.get("bin_dispersion", 1)
@@ -417,24 +480,27 @@ class Instrument(InstrumentConfig):
             ref = np.ones_like(wave)*default
         return ref
 
-    def _get_throughput(self, wave, key):
+    def _get_throughput(self, wave, key, colname='throughput'):
         """
         Wrap self._interp_refdata for getting throughput data
-
+        HST does not and cannot use this method as-is because its high-resolution modes require 64-bit arrays
+        
         Parameters
         ----------
         wave: numpy.ndarray
             Wavelength array onto which the throughput curve is to be interpolated
         key: str
             Key to get filename from self.paths
+        colname: str
+            column name where throughput is stored in table
 
         Returns
         -------
         eff: numpy.ndarray or float
             If ref file exists, return efficiency(wave), else return 1.0
         """
-        eff = self._interp_refdata(wave, key, colname='throughput', default=1.0)
-        return eff
+        eff = self._interp_refdata(wave, key, colname=colname, default=1.0)
+        return np.array(eff, dtype=np.float32)
 
     def _get_disperser(self):
         """
@@ -755,13 +821,13 @@ class Instrument(InstrumentConfig):
 
         return wavepix
 
-    def get_chip_dimensions(self):
+    def get_chip_dimensions(self, wave):
         """
         This function allows us to specify the dimensions of a chip or area on a detector
         that have their own properties.
         
         Use case: 
-        COS FUV detector is comprised of two chips that have their own readnoise and 
+        COS FUV detector is comprised of two chips that have their own
         dark current, and require their own health and safety outputs; the spectrum 
         is produced as a single array with a gap. COS will need to override this function
         to produce that output.
@@ -910,6 +976,7 @@ class Instrument(InstrumentConfig):
             "detector": self.instrument["detector"],
             "aperture": self.instrument["aperture"],
             "disperser": self.instrument["disperser"],
+            "cenwave": self.instrument.get("cenwave", None),
             "filter": self.instrument["filter"],            
             "instmode": self.instrument["mode"],
             "readout_pattern": self.detector.get("readout_pattern", None),
@@ -917,8 +984,7 @@ class Instrument(InstrumentConfig):
             "binning": "b{}".format(self.get_binning()),
             "subarray": self.detector.get("subarray", None),
             "ngroup": "g{}".format(self.detector.get("ngroup", None)),
-            "dark_level": self.detector.get("dark_level", None)
-
+            "dark_level": self.detector.get("dark_level", None),
         }
 
         if parameter_key in item:
@@ -1169,7 +1235,7 @@ class Instrument(InstrumentConfig):
             wmax = self.range[aperture]["wmax"]
 
         # now check wmin and wmax against wave_pix for sanity, if applicable
-        if self.projection_type in ('slitless', 'spec'):
+        if self.projection_type in ('slitless', 'spec', 'slitless_scan'):
             wave_pix = self.get_wave_pix()
             wmin = max(wmin, wave_pix.min())
             wmax = min(wmax, wave_pix.max())
@@ -1185,7 +1251,7 @@ class Instrument(InstrumentConfig):
         Returns
         -------
         proj_type: str
-            The projection type, currently one of 'slitless', 'spec', or 'image'.
+            The projection type, currently one of 'slitless', 'slitless_scan', 'spec', 'image', or 'image_scan'.
         """
         mode = self.mode
         slitless_modes = self.telescope.slitless_modes
@@ -1195,15 +1261,30 @@ class Instrument(InstrumentConfig):
             multiorder_modes = self.telescope.multiorder_modes
         else:
             multiorder_modes = []
+        if hasattr(self.telescope, 'slitless_scan_modes'):
+            slitless_scan_modes = self.telescope.slitless_scan_modes
+        else:
+            slitless_scan_modes = []
+        if hasattr(self.telescope, 'image_scan_modes'):
+            image_scan_modes = self.telescope.image_scan_modes
+        else:
+            image_scan_modes = []
 
         if mode in slitless_modes:
             proj_type = 'slitless'
-        if mode in spec_modes:
+        elif mode in spec_modes:
             proj_type = 'spec'
-        if mode in image_modes:
+        elif mode in image_modes:
             proj_type = 'image'
-        if mode in multiorder_modes:
+        elif mode in multiorder_modes:
             proj_type = 'multiorder'
+        elif mode in slitless_scan_modes:
+            proj_type = 'slitless_scan'
+        elif mode in image_scan_modes:
+            proj_type = 'image_scan'
+        else:
+            raise DataError("Unrecognized projection type for mode {}".format(mode))
+
         return proj_type
 
     def dispersion_axis(self):
@@ -1226,11 +1307,12 @@ class Instrument(InstrumentConfig):
 
         Returns
         -------
-        pix_size: float
-            The pixel scale in arcsec/pixel
+        pix_size: tuple
+            The pixel scale in the y and x directions in arcsec/pixel
         """
         aperture_dict = self.get_aperture_pars()
-        pix_size = aperture_dict['pix']
+        pix_size = aperture_dict['plate_scale']
+
         return pix_size
 
     def get_exposure_time(self):
@@ -1349,3 +1431,52 @@ class Instrument(InstrumentConfig):
 
         spec.flux = flux_oldsampling
         return spec
+
+    def get_scan_length(self):
+        """
+        Get, or compute, the length of a scan for a scanned spec or scan image mode.
+        Users always have to specify the exposure_time (or components sufficient to calculate it); 
+        if they scan, they must provide either scan_length (arcseconds) or scan_rate (arcseconds/sec), 
+        otherwise this code raises an EngineInputError.
+
+        If users specify BOTH a scan_length AND scan_rate, the code raises an EngineInputError if 
+        the values are inconsistent.
+
+        Returns
+        -------
+        scan_length: float
+            length of the scan in arcseconds
+
+        """
+        if "scan_length" in self.the_detector.input_detector:
+            scan_length = self.the_detector.input_detector["scan_length"]
+        elif "scan_rate" in self.the_detector.input_detector:
+            scan_length = self.the_detector.input_detector["scan_rate"] * self.the_detector.exposure_spec.exposure_time
+        else:
+            raise EngineInputError("Cannot define scan without scan length or scan rate.")
+        # And now a sanity check: If all three are defined, are they consistent?
+        if "scan_rate" in self.the_detector.input_detector:
+            # This will be trivially true if we just computed the scan length from the scan rate.
+            if scan_length != (self.the_detector.input_detector["scan_rate"] * self.the_detector.exposure_spec.exposure_time):
+                raise EngineInputError("Scan definition inconsistent: time {}, rate {}, length {}".format(self.the_detector.exposure_spec.exposure_time, self.the_detector.input_detector["scan_rate"], scan_length))
+        return scan_length
+
+    def get_segment_indicies(self, wave):
+        """
+        Generic code to get the boundaries of a chip/stripe/segment in pixel coordinates, from our segment
+        definitions. Functions that need this will override it.
+
+        Parameters
+        ----------
+        wave: np.ndarray
+            The wavelength array of the projected grid
+
+        Returns
+        -------
+        segment_indices: list
+            A list of dictionaries defining the segments in pixel coordinates.
+        """
+
+        segment_indicies = [{"segname": "", "wmin_ind": 0, "wmax_ind": len(wave)}]
+        
+        return segment_indicies
