@@ -11,6 +11,7 @@ import numpy as np
 from .telescope import Telescope
 from .instrument import Instrument
 from .io_utils import read_json
+from .psf_library import PSFLibrary
 from .custom_exceptions import EngineInputError, RangeError, DataError
 
 class JWST(Telescope):
@@ -49,7 +50,8 @@ class JWSTInstrument(Instrument):
            Put the detector keyword in self.instrument['detector']."""
         if "slit" in self.aperture_config[self.get_aperture()]:
             self.instrument['slit'] = self.aperture_config[self.get_aperture()]['slit']
-        self.instrument['detector'] = self.aperture_config[self.get_aperture()]['detector']
+        if "detector" not in self.instrument:
+            self.instrument['detector'] = self.aperture_config[self.get_aperture()]['detector']
 
 class NIRSpec(JWSTInstrument):
 
@@ -86,6 +88,7 @@ class NIRSpec(JWSTInstrument):
                     raise DataError(value=msg)
             else:
                 self.gap = self.read_config_param(slit_config, "gap")
+        self._loadpsfs()
 
     def get_wave_range(self):
         """
@@ -185,6 +188,9 @@ class NIRSpec(JWSTInstrument):
         overloaded because the internal optical throughput is
         different for the NIRSpec IFU compared to MOS and Fixed Slit.
 
+        This also multiplies the internal efficiency by the correction
+        factor that showed up after commissioning.
+
         Parameters
         ----------
         wave: numpy.ndarray
@@ -203,21 +209,73 @@ class NIRSpec(JWSTInstrument):
             msg = "Internal efficiency not configured for NIRSpec mode %s." % self.mode
             raise EngineInputError(value=msg)
 
-        return eff
+        # correction factors for the filter+disperser combinations
+
+        # generate a valid key only for the 3 modes that support correction factors
+        # (this will change when more cf files are delivered)
+        str_mode = "Nomode" # causes a fall thru to np.ones correction factor
+        if "ifu" in self.mode:
+            str_mode = "ifu"
+        if "fixed_slit" in self.mode or "bots" in self.mode:
+            str_mode = "fs"
+        elif "mos" in self.mode:
+            str_mode = "mos"
+
+        disperser = self.instrument['disperser']
+        if disperser is None:
+            disperser = 'prism'
+        filter = self.instrument['filter']
+        if filter is None:
+            filter = 'clear'
+
+        key = "cf_%s_%s_%s" % (str_mode, filter, disperser)
+
+        corr = self._get_throughput(wave, key, colname="obs/exp")
+
+        return eff * corr
 
 class NIRCam(JWSTInstrument):
 
-    def _loadpsfs(self):
-        """
-        For the bar-shaped coronagraphy masks we need to load the psf_library on a per-filter basis.
-        The PSF files have the aperture and filter smooshed into one string so use that as the key.
-        """
-        if self.instrument['aperture'] in ('masklwb', 'maskswb'):
-            psf_key = "%s%s" % (self.instrument['aperture'], self.instrument['filter'])
-        else:
-            psf_key = self.instrument['aperture']
+    """
+    Need to override __init__ to handle the complexity of the multiple coronagraphy apertures that we want to
+    (optionally) hide from users.
+    """
 
-        self.psf_library = self._load_psf_library(psf_key)
+    def __init__(self, mode=None, config={}, webapp=False, **kwargs):
+
+        if mode == "coronagraphy":
+            # If we have one of the non-suffixed apertures, we must apply the appropriate detector suffix.
+            # This code would turn mask210rsw into mask210rswlw if you use one suffixed aperture with the
+            # other detector but that shouldn't work anyway. Nevertheless, let's put in an informative error
+            # message.
+            for detector in ["sw", "lw"]:
+                if config["instrument"]["aperture"][-2:] == detector and config["instrument"]["detector"] != detector:
+                    raise EngineInputError(f'Inconsistent selection of detector {config["instrument"]["detector"]} and aperture {config["instrument"]["aperture"]}')
+
+            if config["instrument"]["detector"] != config["instrument"]["aperture"][-2:]:
+                config["instrument"]["aperture"] = f"{config['instrument']['aperture']}{config['instrument']['detector']}"
+        elif mode == "target_acq":
+            # target_acq also uses coronagraphic apertures, but only has a 1:1 mapping.
+            aperture_map = {"mask210r": "mask210rsw", "mask335r": "mask335rlw", "mask430r": "mask430rlw", "maskswb": "maskswbsw", "masklwb": "masklwblw", "sw": "sw", "lw": "lw",
+                            "mask210rsw": "mask210rsw", "mask335rlw": "mask335rlw", "mask430rlw": "mask430rlw", "maskswbsw": "maskswbsw", "masklwblw": "masklwblw"}
+            config["instrument"]["aperture"] = aperture_map[config["instrument"]["aperture"]]
+
+        JWSTInstrument.__init__(self, mode=mode, config=config, webapp=webapp, **kwargs)
+
+        self._loadpsfs()
+
+    def _load_psf_library(self,psf_key):
+
+        bar_offset=0
+        if "masklwbsw" in self.get_aperture() or "maskswblw" in self.get_aperture():
+            bar_offset = self.bar_offsets[self.instrument['paired_filter']]
+        elif "masklwblw" in self.get_aperture() or "maskswbsw" in self.get_aperture():
+            bar_offset = self.bar_offsets[self.instrument['filter'].split("_")[0]]
+
+        psf_path = os.path.join(self.ref_dir, "psfs")
+        library = PSFLibrary(self.get_wave_range(), path=psf_path, psf_key=psf_key, x_offset=bar_offset)
+
+        return library
 
     def get_filter_eff(self, wave):
         """
@@ -320,8 +378,6 @@ class NIRCam(JWSTInstrument):
             Width of bar at X in arcsec
         """
         filt = self.instrument['filter']
-        if (len(filt.split('_')) > 1) and (filt.split('_')[1] == 'nd'): # if this is a neutral density combination, use the main filter to get the bar width
-            filt = filt.split('_')[0]
         if filt not in self.bar_offsets:
             msg = "Invalid filter, %s, for MASKLWB/MASKSWB." % filt
             raise DataError(value=msg)
@@ -357,14 +413,13 @@ class NIRCam(JWSTInstrument):
 class NIRISS(JWSTInstrument):
 
     """
-    Need to over-ride get_internal_eff() because there are two different wheels in NIRISS.
-    The 'filter wheel' has a clear position with 100% transmission. The 'pupil wheel'
-    (which somewhat confusingly also contains filters) has a clear position with some
-    obstruction (the Pupil Alignment Reference - PAR). So if the active filter is in the
-    'filter wheel', the transmission takes a hit from the PAR in the pupil wheel.
+    Need to override _loadpsfs because the long wavelength filters go through a different mask (CLEARP) than
+    the short-wavelength filters, and require a different set of PSFs that does overlap in wavelength.
 
-    Also need to override dispersion_axis() because the BG150C grating disperses along the
-    Y axis vs. the X axis like every other JWST disperser.
+    Also need to set up multiorder extraction masks for SOSS, and order-specific wavelength traces.
+
+    Also need to override dispersion_axis() because the GR150C and GR700XD gratings disperse along the Y axis
+    vs. the X axis like every other JWST disperser.
     """
 
     def __init__(self, mode=None, config={}, webapp=False, **kwargs):
@@ -373,6 +428,7 @@ class NIRISS(JWSTInstrument):
         self.api_ignore = ['max_saturated_pixels', 'min_snr_threshold', 'aperture_size']
 
         JWSTInstrument.__init__(self, mode=mode, config=config, webapp=webapp, **kwargs)
+        self._loadpsfs()
 
     def dispersion_axis(self):
         """
@@ -514,6 +570,10 @@ class MIRI(JWSTInstrument):
     The MIRI internal efficiency, detector readout patterns, etc. are more complex, and different than the other instruments,
     so some methods are redefined.
     """
+    def __init__(self, mode=None, config={}, webapp=False, **kwargs):
+
+        JWSTInstrument.__init__(self, mode=mode, config=config, webapp=webapp, **kwargs)
+        self._loadpsfs()
 
     def _loadpsfs(self):
         """
@@ -558,33 +618,6 @@ class MIRI(JWSTInstrument):
 
         return var_fudge
 
-    def get_dichroic(self, type, level):
-        """
-        MIRI dichroic reference files are keyed by type (refl or trans), level, and disperser
-
-        Parameters
-        ----------
-        type: str
-            Either 'refl' for reflective or 'trans' for transmitting
-        level: int
-            Level 1, 2, or 3
-
-        Returns
-        -------
-        key: str
-            Key pointing to the dichroic efficiency data in self.paths
-        """
-        disperser = self.instrument['disperser']
-        if disperser == 'short':
-            mrs_setting = 'a'
-        elif disperser == 'medium':
-            mrs_setting = 'b'
-        elif disperser == 'long':
-            mrs_setting = 'c'
-
-        key = "sd" + str(level) + mrs_setting + "_" + type
-        return key
-
     def get_internal_eff(self, wave):
         """
         Calculate MIRI internal efficiency which is rather more complicated than the other instruments
@@ -604,52 +637,7 @@ class MIRI(JWSTInstrument):
         mirror_cont = self.mirror_cont
         n_refl = self.n_reflections[aperture]
         refl_eff = mirror_eff ** n_refl
-        if self.mode in ['mrs', 'mrs_ts']:
-            if aperture == 'ch1':
-                d1r_key = self.get_dichroic('refl', 1)
-                d1r_eff = self._get_throughput(wave, d1r_key)
-                internal_eff = d1r_eff * refl_eff
-
-            elif aperture == 'ch2':
-                d1t_key = self.get_dichroic('trans', 1)
-                d2r_key = self.get_dichroic('refl', 2)
-                d1t_eff = self._get_throughput(wave, d1t_key)
-                d2r_eff = self._get_throughput(wave, d2r_key)
-                internal_eff = d1t_eff * d2r_eff * refl_eff
-
-            elif aperture == 'ch3':
-                d1t_key = self.get_dichroic('trans', 1)
-                d2t_key = self.get_dichroic('trans', 2)
-                d3r_key = self.get_dichroic('refl', 3)
-                d1t_eff = self._get_throughput(wave, d1t_key)
-                d2t_eff = self._get_throughput(wave, d2t_key)
-                d3r_eff = self._get_throughput(wave, d3r_key)
-                internal_eff = d1t_eff * d2t_eff * d3r_eff * refl_eff
-
-            elif aperture == 'ch4':
-                d1t_key = self.get_dichroic('trans', 1)
-                d2t_key = self.get_dichroic('trans', 2)
-                d3t_key = self.get_dichroic('trans', 3)
-                d1t_eff = self._get_throughput(wave, d1t_key)
-                d2t_eff = self._get_throughput(wave, d2t_key)
-                d3t_eff = self._get_throughput(wave, d3t_key)
-                internal_eff = d1t_eff * d2t_eff * d3t_eff * refl_eff
-
-            else:
-                raise EngineInputError(value='Invalid aperture for MIRI: {0:}'.format(aperture))
-
-        elif (self.mode == 'coronagraphy') or (self.mode == "target_acq"):
-            # The 4QPM coronagraph masks are made of germanium, and need transmission components added.
-            if (aperture == 'fqpm1065') or (aperture == 'fqpm1140'):
-                ar1_eff = self._get_throughput(wave, 'ge_ar1_trans')
-                internal_eff = ar1_eff * refl_eff
-            elif (aperture == 'fqpm1550'):
-                ar2_eff = self._get_throughput(wave, 'ge_ar2_trans')
-                internal_eff = ar2_eff * refl_eff
-            else:
-                internal_eff = np.zeros_like(wave) + refl_eff
-        else:
-            internal_eff = np.zeros_like(wave) + refl_eff
+        internal_eff = refl_eff
 
         # mirror contamination factor
         internal_eff = internal_eff * mirror_cont
@@ -671,7 +659,7 @@ class MIRI(JWSTInstrument):
             Disperser efficiency as a function of wave
         """
         if self.instrument['disperser'] is not None:
-            key = self.instrument['aperture']
+            key = "{}_{}".format(self.instrument['aperture'],self.instrument['disperser'])
             disperser_eff = self._get_throughput(wave, key)
         else:
             disperser_eff = 1.
