@@ -3,6 +3,7 @@ from __future__ import division, absolute_import
 
 import os
 import numpy as np
+import scipy.integrate as integ
 import astropy.units as u
 
 # suppress stsynphot's unnecessary warning (we'll supply our own Vega spectrum)
@@ -20,7 +21,7 @@ from .custom_exceptions import DataConfigurationError, EngineInputError, Synphot
 from .pandeia_warnings import normalization_warning_messages as warning_messages
 from .utils import get_key_list, get_dict_from_keys, recursive_subclasses, merge_data
 from .instrument_factory import InstrumentFactory
-from .constants import PANDEIA_WAVEUNITS, PANDEIA_FLUXUNITS, UNIT_MAP
+from .constants import PANDEIA_WAVEUNITS, PANDEIA_FLUXUNITS, UNIT_MAP, MIN_CLIP
 
 from . import io_utils as io
 from . import config as cf
@@ -289,15 +290,21 @@ class NormalizePhotsys(Normalization):
 
         sp = syn.spectrum.SourceSpectrum(Empirical1D, points=wave, lookup_table=flux)
         bp = self._get_bandpass(wave)
-        try:
-            sp_rn = sp.normalize(self.norm_flux, band=bp, vegaspec=syn.spectrum.SourceSpectrum.from_vega())
-        except Exception as e:
-            msg = "Error using Synphot to perform bandpass normalization in %s" % str(bp)
-            if self.webapp:
-                msg += "(%s)" % type(e)
-            else:
-                msg += repr(e)
-            raise SynphotError(value=msg)
+        # Check if the flux is less than 1e-10 mJy
+        if integ.trapz(sp(bp.waveset, flux_unit=PANDEIA_FLUXUNITS).value) < MIN_CLIP:
+            key = 'normalized_to_zero_flux_bandpass'
+            self.warnings[key] = warning_messages[key]
+            sp_rn = sp
+        else:
+            try:
+                sp_rn = sp.normalize(self.norm_flux, band=bp, vegaspec=syn.spectrum.SourceSpectrum.from_vega())
+            except Exception as e:
+                msg = "Error using Synphot to perform bandpass normalization in %s" % str(bp)
+                if self.webapp:
+                    msg += "(%s)" % type(e)
+                else:
+                    msg += repr(e)
+                raise SynphotError(value=msg)
 
         return sp_rn.waveset.to(PANDEIA_WAVEUNITS), sp_rn(sp_rn.waveset, flux_unit=PANDEIA_FLUXUNITS)
 
