@@ -45,6 +45,7 @@ strategy_warning_messages.update(standard_warning_messages)
 # warning messages specific to Normalization and its sub-classes
 normalization_warning_messages = {
     "normalized_to_zero_flux": "Zero flux at reference wavelength. Spectrum left unscaled.",
+    "normalized_to_zero_flux_bandpass": "Zero flux in bandpass. Spectrum left unscaled.",
     "unsupported_normalization_bandpass": "Bandpass specification, %s, not currently supported, but may work."
 }
 normalization_warning_messages.update(standard_warning_messages)
@@ -68,7 +69,8 @@ astrospectrum_warning_messages = {
     "wavelength_truncated_red": "Spectrum red limit, %.2f, does not extend to instrument configuration red limit, %.2f.",
     "scene_range_truncated": "Combined wavelength range of scene [%.2f, %.2f] less than instrument configuration's range [%.2f, %.2f].",
     "spectrum_missing_blue": "Spectrum [%.2f, %.2f] does not extend to instrument configuration blue limit, %.2f.",
-    "spectrum_missing_red": "Spectrum [%.2f, %.2f] does not extend to instrument configuration red limit, %.2f."
+    "spectrum_missing_red": "Spectrum [%.2f, %.2f] does not extend to instrument configuration red limit, %.2f.",
+    "filter_leak": "This observation has a significant filter leak. {0:.3f} percent of the signal of one of the sources comes from outside the prescribed filter bandpass limits and is not included in the results.",
 }
 astrospectrum_warning_messages.update(standard_warning_messages)
 
@@ -80,16 +82,12 @@ report_warning_messages = {
     "cube_partial": "There are {0:} total partially saturated pixels in the data cube.",
     "cube_full": "There are {0:} total fully saturated pixels in the data cube.",
     "bad_waveref": "Specified wavelength, {0:.2f}, out of range [{1:.2f}, {2:.2f}]. Using {3:.2f} to select diagnostic planes instead.",
+    "no_waveref": "No specified wavelength for spectral mode. Using {0:.2f} to select diagnostic planes.",
     "ta_max_saturated_pixels": "<font color=red><b>TA MAY FAIL</b></font>: Number of fully saturated pixels {0:} in centroid" \
                     " box exceeds the maximum number of {1:} pixels allowed to ensure a successful target acquisition. "\
                     "Recommend to adjust Detector Setup and/or Instrument Setup accordingly.",
     "ta_snr_threshold": "<font color=red><b>TA MAY FAIL</b></font>: calculated SNR {0:.2f} must exceed {1:.2f} to ensure TA success.",
     "partial_saturated_ami": "Partial saturation:\n There are {0:1d} pixels saturated* at the end of a ramp. Partial ramps may still be used in some cases. *(See footnote in the Saturation image tab.)",
     "full_saturated_ami": "Full saturation:\n There are {0:1d} pixels saturated* at the end of the first group. These pixels cannot be recovered. *(See footnote in the Saturation image tab.)",
-    "brightest_native_pixel_ccd": "CCD Full Well limits: total counts per native pixel {0:.2f} exceeds bright limit {1:.2f} for selected gain",
-    "brightest_bin_pixel_ccd": "CCD Full Well limits: total counts per binned pixel {0:.2f} exceeds bright limit {1:.2f} for selected gain",
-    "brightest_pixel_mama": "MAMA Bright limits: count rate of brightest pixel {0:.2f} exceeds bright limit {1:.2f}",
-    "detector_total_rate_mama": "MAMA Bright limits: total count rate per image {0:.2f} exceeds bright limit {1:.2f}",
-    "buffer_time": "MAMA Bright limits: Because buffer fill time is less than {0:.2f} sec, this observation can only be performed in ACCUM mode."
 }
 report_warning_messages.update(standard_warning_messages)
