@@ -14,7 +14,8 @@ def ProjectionFactory(projection_type):
     Parameters
     ----------
     projection_type: string
-        String with four valid values: "Image", "Spec", "Slitless", "Multiorder".
+        String with six valid values: "image", "image_scan", "spec", "slitless", 
+        "slitless_scan", "multiorder".
     """
 
     types = recursive_subclasses(Projection)
@@ -81,7 +82,8 @@ class ImageProjection(Projection):
 
         return signal.grid, signal.wave_pix, signal.pixgrid_list[0]
 
-
+class Image_ScanProjection(ImageProjection):
+    pass
 class SpecProjection(Projection):
 
     def _2d(self, extracted, saturation, signal):
@@ -143,6 +145,8 @@ class SlitlessProjection(Projection):
 
         return grid, wave_pix, pix_grid
 
+class Slitless_ScanProjection(SlitlessProjection):
+    pass
 
 class MultiorderProjection(SlitlessProjection):
 
