@@ -3,6 +3,7 @@ from __future__ import division, absolute_import
 
 from .telescope import Telescope
 from .instrument import Instrument
+from .custom_exceptions import EngineInputError
 
 class Roman(Telescope):
 
@@ -42,6 +43,11 @@ class WFI(RomanInstrument):
     """
     Currently, the Roman WFI requires only one method beyond those provided by the generic Instrument class
     """
+    def __init__(self, mode=None, config={}, webapp=False, **kwargs):
+
+        RomanInstrument.__init__(self, mode=mode, config=config, webapp=webapp, **kwargs)
+        self._loadpsfs()
+
     def _loadpsfs(self):
         """
         Short-wavelength filters need PSFs with the skinny pupil mask.
