@@ -45,7 +45,7 @@ def spectrum_resample(flux, orig_wave, new_wave, mask_val=np.nan):
             # to PHOTLAM (an flam unit), binning, and converting back to mJy. (JETC-300)
             # See similar code in helpers/background/butler.py bg_resample.
             spec = syn.spectrum.SourceSpectrum(Empirical1D, points=orig_wave, lookup_table=flux)
-            dummy_filter = syn.spectrum.SpectralElement(ConstFlux1D, amplitude=1)
+            dummy_filter = syn.spectrum.SpectralElement(Empirical1D, points=orig_wave, lookup_table=np.ones_like(flux))
             # We can't just resample the flux, because that performs linear interpolation (Empirical1D model is based on
             # the astropy Tabular1D model, which according to this page:
             # https://docs.astropy.org/en/stable/modeling/reference_api.html#astropy.modeling.tabular.Tabular1D just does
@@ -134,6 +134,38 @@ def merge_data(*dicts):
     return updated
 
 
+def apply_default_only(input_dict):
+    """
+    Recursively apply the special default_only parameters
+
+    This is a special section for handling locations where there are two different keywords that can't
+    coexist, like specifying scan_length XOR scan_rate in a scan calculation. The pandeia.engine Instrument
+    default loader can't add both, and doesn't know which one the user might choose, so it must require
+    neither. But at least one MUST be present, so we need to specify one to add when we build a calculation
+    (here).
+
+    Paramters
+    ---------
+    input_dict: dictionary
+        A dictionary from somewhere in the engine input dictionary
+
+    Returns
+    -------
+    input_dictionary: dictionary
+        The same dictionary with "default_only" values moved into the main dictionar(ies), and the default_only
+        subdict removed.
+
+    """
+    if "default_only" in input_dict:
+        input_dict.update(input_dict['default_only'])
+        # and now delete the extraneous dictionary
+        del input_dict['default_only']
+    for item in input_dict:
+        if isinstance(input_dict[item], dict):
+            input_dict[item] = apply_default_only(input_dict[item])
+    return input_dict
+
+
 def get_key_list(key, separator=default_separator):
     """
     Create list of keys from a flattened key to set/retrieve value from nested dict
